#-*- Makefile -*-

require DE_EPAGES::Core::API::Makefile;
__DATA__

!include host-$(HOSTNAME).mk

PATCH_TAG	= patch_6_16_1
PATCH_VERSION	= 6.16.1
DE_EPAGES_DIR	= $(EPAGES_CARTRIDGES)\DE_EPAGES
DATA_DIR	= $(EPAGES_SHARED)\Patches\var-$(PATCH_TAG)
LOG_DIR		= $(DATA_DIR)\logs\$(HOSTNAME)
TGT		= $(DATA_DIR)\tgt\$(HOSTNAME)

CREATE_PATCHDB	= $(PERL) Scripts/createPatchDB.pl
DROP_PATCHDB	= $(PERL) Scripts/dropPatchDB.pl

PATCH_SERVICE	=
PATCH_REQROUTER	=
PATCH_MONITOR	=
STOP_EPAGES	=
START_EPAGES	=
RESTART_EPAGES	= 1

!OS_SPECIFIC_DEFINES
PRL		= $(PERL) -IInclude
INCLUDES	= -I$(EPAGES_CARTRIDGES)/.version/$(PATCH_VERSION)
!ifdef WINDOWS
PATCH_WEBADAPTER= 1
IF_EXIST	= if exist
IF_NOT_EXIST	= if not exist
DO		=
RMDIR		= rmdir /S /Q
DIFF		= gdiff
MAKE		= nmake -nologo
!else
PATCH_WEBADAPTER=
SHELL		= /bin/sh
IF_EXIST	= [ ! -e
IF_NOT_EXIST	= [ -e
DO		= ] ||
RMDIR		= rm -rf
DIFF		= diff
EPAGES_PROJECT	?= epages6
MAKE		= make
!BEGIN_VERBATIM
PATCH_RELEASE_PATH=$(shell ls -d `pwd`/sh/6*/EPAGES_RELEASE/var/epages 2>/dev/null | tac | tr '\n' ':')
!END_VERBATIM
RELEASE_PATH	= PATH=$(PATCH_RELEASE_PATH)/var/epages:$$PATH
!endif

LOOP_STORES	= $(PRL) Scripts/LoopStores.pl
START_SERVICE   = $(PRL) $(DE_EPAGES_DIR)/Installer/Scripts/startService.pl -ifexists
STOP_SERVICE    = $(PRL) $(DE_EPAGES_DIR)/Installer/Scripts/stopService.pl -ifexists
CREATE_CTMPLS   = $(PRL) $(DE_EPAGES_DIR)/Presentation/Scripts/createCtmpls.pl

CALL_PATCH_HANDLER = $(PERL) Scripts/call-PatchHandler.pl
SET_PATCHDB_BASIC_ATTRIBUTES = $(PERL) Scripts/setPatchDBBasicAttributes.pl
INITIALIZE_PATCH_SYSTEM = $(PERL) -IIncludeHandler Scripts/initializePatchSystemInfoDB.pl

STORE		= dummy

.force:

# we always need following target for patch.pl:
get_var: .force
	@echo LOG_DIR=$(LOG_DIR)
	@echo TGT_DIR=$(DATA_DIR)\tgt
	@echo TGT=$(TGT)

# how to install the epages patch (see install_en.txt)
# ****************************************************
#-------------------------------------------------------------------------------------------
#    | execute                             | on  UNIX                  | on Windows        |
#-------------------------------------------------------------------------------------------
# 1. | set environment                     | . /etc/default/epages6    |                   |
# 2. | create makefile (needs epages perl) | perl Makefile.PL          | perl Makefile.PL  |
# 3. | pre-installation tasks              | make preinstall           | nmake preinstall  |
# 4. | set environment                     | . /etc/default/epages6    |                   |
# 5. | installation tasks* (see NOTE)      | make install              | nmake install     |
#-------------------------------------------------------------------------------------------
# *) if epages runs on more than 1 computer, execute this step only on your main application server

create_patch_system_info_db:
	$(CREATE_PATCHDB)

drop_patch_system_info_db:
	$(DROP_PATCHDB)

reinitialize:
	$(MAKE) drop_patch_system_info_db
	$(MAKE) create_patch_system_info_db
	$(DEL) $(DATA_DIR)\tgt\initialize
	$(MAKE) initialize

initialize: $(DATA_DIR)\tgt\initialize
$(DATA_DIR)\tgt\initialize:
	$(MAKE) drop_patch_system_info_db
!ifndef WINDOWS
	./Scripts/install-hotfixes.sh -stop $(PATCH_VERSION)
!endif
	$(MAKE) create_patch_system_info_db
	$(INITIALIZE_PATCH_SYSTEM) -patchsystemdbsourcedir Database/mysql
	$(MAKE) touch DIR=$(@D) ADDF=$(@F)

download:
	$(CALL_PATCH_HANDLER) -handler PatchDownloadHandler

prepare: $(DATA_DIR)\tgt\initialize $(DATA_DIR)\tgt\prepare
$(DATA_DIR)\tgt\prepare:
	$(CALL_PATCH_HANDLER) -handler PatchDownloadHandler
	$(CALL_PATCH_HANDLER) -handler PatchPrepareHandler
	$(MAKE) touch DIR=$(@D) ADDF=$(@F)

preinstall: prepare $(TGT)\preinstall
$(TGT)\preinstall:
!ifndef WINDOWS
	./Scripts/install-hotfixes.sh -stop $(PATCH_VERSION)
!endif
	$(MAKE) stop_epages update_os httpd\close stop_cron
	$(DEL) $(DATA_DIR)\tgt\posthosts.txt
	$(PRL) Scripts/checkScheduler.pl -hostname $(HOSTNAME)
	$(AWK) "/^$(HOSTNAME)$$/{i=1}END{exit i!=1}" $(DATA_DIR)\tgt\prehosts.txt 2>$(NUL) || echo $(HOSTNAME)>> $(DATA_DIR)\tgt\prehosts.txt
	$(MAKE) touch DIR=$(@D) ADDF=$(@F)

install: preinstall close_stores patch_files restart_epages alpha_update_os
	$(CALL_PATCH_HANDLER) -handler PatchInstallHandler
	$(MAKE) update_cartridge_dir update_ctmpls
	echo $(HOSTNAME)> $(DATA_DIR)\tgt\alphahost.txt
	$(MAKE) touch DIR=$(DATA_DIR)\tgt ADDF=install
	$(PERL) -MFile::Compare -e "exit not compare('$(DATA_DIR)\tgt\alphahost.txt','$(DATA_DIR)\tgt\prehosts.txt')"\
 || $(MAKE) postinstall IS_ALPHA=1 POSTINSTALL_COMPLETE=1
	$(MAKE) perm

cleanup:
	$(CALL_PATCH_HANDLER) -handler PatchPostInstallHandlerCleanUp

recreate_solr_search_index:
	$(CALL_PATCH_HANDLER) -handler PatchPostInstallHandlerRemoteSearch

set_patchdb_basic_attributes:
	$(SET_PATCHDB_BASIC_ATTRIBUTES) -key $(KEY) -value $(VALUE)

postinstall:
	$(CALL_PATCH_HANDLER) -handler PatchPostInstallHandler
!ifdef IS_ALPHA
!ifneq ("$(POSTINSTALL_COMPLETE)", "1")
	@echo "Postinstall on alpha server failed: First postinstall all other servers."
	@$(PERL) -e "exit 1;"
!endif
!else
	$(MAKE) patch_files
	$(MAKE) perm
!endif
!ifndef WINDOWS
	$(RELEASE_PATH) movelink.sh /etc/default/$(EPAGES_PROJECT).lnk
	$(RELEASE_PATH) movelink.sh /etc/init.d/$(EPAGES_PROJECT).lnk
!endif
	$(MAKE) $(DATA_DIR)\tgt\install start_cron restart_httpd
!ifdef IS_ALPHA
	$(MAKE) open_stores
!ifndef NO_RESTART
	$(MAKE) restart_epages
!endif
!ifndef FLEX_INSTALLATION
	$(CALL_PATCH_HANDLER) -handler PatchPostInstallHandlerSynchronizeSite
!endif
	$(CALL_PATCH_HANDLER) -handler PatchPostInstallHandlerRemoveCartridges
	$(PRL) Scripts/findOverloadedTemplates.pl -store all -patchdir . -initial $(INITIAL_VERSION)
!else
	$(MAKE) start_epages
!endif
!ifndef WINDOWS
	./Scripts/install-hotfixes.sh -start $(PATCH_VERSION)
!endif
	echo $(HOSTNAME)>> $(DATA_DIR)\tgt\posthosts.txt

patch_files:
!ifeq ("$(CHANGE_CONFIG)", "1")
	$(MAKE) Config\patch
!endif
!ifeq ("$(PATCH_SERVICE)", "1")
!ifeq ("$(CHANGE_SERVICE)", "1")
	$(MAKE) Service\patch
!endif
!endif
!ifeq ("$(PATCH_REQROUTER)", "1")
!ifeq ("$(CHANGE_REQROUTER)", "1")
	$(MAKE) RequestRouter\patch
!endif
!endif
!ifeq ("$(PATCH_WEBADAPTER)", "1")
!ifeq ("$(CHANGE_WEBADAPTER)", "1")
	$(MAKE) WebAdapter\patch
!endif
!endif
!ifeq ("$(CHANGE_BIN)", "1")
	$(MAKE) bin\patch
!endif

# targets (patch files)
# *********************

# patch: bin
bin\patch: $(EPAGES)\bin\.$(PATCH_VERSION)-patch
$(EPAGES)\bin\.$(PATCH_VERSION)-patch:
	$(PRL) Scripts/patch_files.pl -patch bin -version $(PATCH_VERSION)

# patch: Service
Service\patch: $(EPAGES)\Service\.$(PATCH_VERSION)-patch
$(EPAGES)\Service\.$(PATCH_VERSION)-patch:
	$(PRL) Scripts/patch_files.pl -patch Service -version $(PATCH_VERSION)

# patch: RequestRouter
RequestRouter\patch: $(EPAGES)\RequestRouter\.$(PATCH_VERSION)-patch
$(EPAGES)\RequestRouter\.$(PATCH_VERSION)-patch:
	$(PRL) Scripts/patch_files.pl -patch RequestRouter -version $(PATCH_VERSION)

# patch: WebAdapter
WebAdapter\patch: $(EPAGES)\WebAdapter\.$(PATCH_VERSION)-patch
$(EPAGES)\WebAdapter\.$(PATCH_VERSION)-patch:
	$(PRL) Scripts/patch_files.pl -patch WebAdapter -version $(PATCH_VERSION)

# patch: Config
Config\patch: $(EPAGES_CONFIG)\.$(PATCH_VERSION)-patch
$(EPAGES_CONFIG)\.$(PATCH_VERSION)-patch:
	$(PRL) Scripts/PatchConfig.pl -source Shared/Config -target "$(EPAGES_CONFIG)" -md5sum md5sum.ini -version $(PATCH_VERSION)

# targets (patch database)
# ************************

close_stores: loop_stores.mak
	$(MAKE) make_makefile
!ifdef EPAGES_DATABASE_SITE
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) close_store STORE=Site STORE_TYPE=Site
!endif
	$(MAKE) -f loop_stores.mak close_stores TGT="$(TGT)"
close_store:
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) close_store STORE=$(STORE) STORE_TYPE=$(STORE_TYPE)
	$(DEL) $(TGT)\open_store\$(STORE)

open_stores: loop_stores.mak
	$(MAKE) make_makefile
!ifdef EPAGES_DATABASE_SITE
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) open_store STORE=Site STORE_TYPE=Site
!endif
	$(MAKE) -f loop_stores.mak open_stores TGT="$(TGT)"
open_store:
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) open_store STORE=$(STORE) STORE_TYPE=$(STORE_TYPE)
	$(DEL) $(TGT)\close_store\$(STORE)

restore_styles: loop_stores.mak
	$(MAKE) -f loop_stores.mak restore_styles TGT="$(TGT)"
restore_style:
	$(PRL) $(DE_EPAGES_DIR)/Design/Scripts/invalidateAllStyles.pl -storename $(STORE)

update_ctmpls: loop_stores.mak
!ifdef EPAGES_DATABASE_SITE
!ifdef CTMPL_TXT
	$(CREATE_CTMPLS) -storename Site -listfile ctmpl-Site.txt -nooutput
!else
	$(CREATE_CTMPLS) -storename Site -delete -nooutput
!endif
!ifdef DEL_SAVED_INCLUDES
	$(IF_EXIST) $(EPAGES_STATIC)\Site\SavedIncludes $(DO) $(RMDIR) $(EPAGES_STATIC)\Site\SavedIncludes
!endif
!endif
	$(MAKE) -f loop_stores.mak update_ctmpls TGT="$(TGT)"
update_ctmpl:
!ifdef CTMPL_TXT
	$(CREATE_CTMPLS) -storename $(STORE) -listfile ctmpl-$(STORE).txt -nooutput
!else
	$(CREATE_CTMPLS) -storename $(STORE) -delete -nooutput
!endif
!ifdef DEL_SAVED_INCLUDES
	$(IF_EXIST) $(EPAGES_STATIC)\$(STORE)\SavedIncludes $(DO) $(RMDIR) $(EPAGES_STATIC)\$(STORE)\SavedIncludes
!endif

test_styles: loop_stores.mak
	$(MAKE) -f loop_stores.mak test_styles TGT="$(TGT)"
test_style:
	$(PRL) $(DE_EPAGES_DIR)/Installer/Scripts/compareCartridgeData.pl -storename $(STORE) -type $(STORE_TYPE)

make_makefile:
	$(CD) $(DE_EPAGES_DIR) && $(PRL) Makefile.PL

loop_stores.mak:
	$(LOOP_STORES) -mak makefile> $@.out
	$(MOVE) $@.out $@

update_cartridge_dir:
!ifndef WINDOWS
!BEGIN_VERBATIM
	chown -R ep_appl:epages "$(EPAGES_CARTRIDGES)/"
	find "$(EPAGES_CARTRIDGES)" -type f -name '*.??.xml' -exec chmod 644 {} \;
	chmod 644 "$(EPAGES_CARTRIDGES)"/*/*/Data/Public/BO/*.??.dot
!END_VERBATIM
!endif

# auxiliary targets
# *****************

stop_cron:
	$(IF_NOT_EXIST) "$(DATA_DIR)"\tgt\prehosts.txt $(DO) $(PRL) Scripts/EditSchedulerConf.pl -stop $(CRON_FILES) -delete "FactFinderInit FactFinderExportProducts FactFinderExportProductsTriggered"
	$(MAKE) stop_sic

start_cron:
	$(PRL) Scripts/EditSchedulerConf.pl -start $(CRON_FILES)
	$(START_SERVICE) cron
	$(MAKE) start_sic

start_sic: $(TGT)\start_sic
$(TGT)\start_sic:
!ifeq ("$(CHANGE_SIC)", "1")
	$(START_SIC)
	$(MAKE) touch DIR=$(@D) ADDF=$(@F)
!endif

stop_sic:
!ifeq ("$(CHANGE_SIC)", "1")
	$(STOP_SIC)
	$(DEL) $(TGT)\start_sic
!endif

stop_epages: .force
!ifeq ("$(STOP_EPAGES)", "1")
	$(STOP_SERVICE) epages
!ifndef WINDOWS
	if pgrep RequestRouter ; then $(ETC_INITD) stop_rr || pkill -9 RequestRouter ; fi
!endif
!endif

start_epages: .force
!ifeq ("$(START_EPAGES)", "1")
	$(START_SERVICE) epages
!endif

restart_epages:
!ifeq ("$(RESTART_EPAGES)", "1")
!ifdef IS_ALPHA
	$(PRL) $(DE_EPAGES_DIR)/WebInterface/Scripts/shutdownAppSrv.pl -sleep 0
!endif
!endif

httpd\close httpd\open:
!ifndef WINDOWS
	$(ETC_INITD) start_httpd $(@F)
!endif

restart_httpd:
!ifndef WINDOWS
	$(MAKE) httpd\open
!endif

alpha_update_os:
!ifndef WINDOWS
	$(RELEASE_PATH) ENABLE_REPO=epages-update-$(PATCH_VERSION) install-RPM.sh --update-only epages-j
	$(RELEASE_PATH) ENABLE_REPO=epages-update-$(PATCH_VERSION) install-RPM.sh --update-only epages-hotfix-base
!endif

update_os:
!ifndef WINDOWS
	$(RELEASE_PATH) install-RPM.sh graphviz mp3splt at
	$(RELEASE_PATH) ENABLE_REPO=epages-arch install-RPM.sh --update epages-nodejs
	$(RELEASE_PATH) movelink.sh /etc/default/$(EPAGES_PROJECT)
	$(RELEASE_PATH) movelink.sh /etc/init.d/$(EPAGES_PROJECT)
	$(RELEASE_PATH) patch-sh.sh -d sh -i $(INITIAL_INTERNAL_VERSION)
!endif

restore_os:
!ifndef WINDOWS
	$(RELEASE_PATH) patch-sh.sh -d sh -R -i $(INITIAL_INTERNAL_VERSION)
!endif

perm:
!ifndef WINDOWS
	-$(ETC_INITD) perm admin
!endif

touch:
	$(MKDIR) $(DIR)
	$(ECHONL) > $(DIR)\$(ADDF)

del_backup_files:
	$(IF_EXIST) "$(DATA_DIR)\tgt" $(DO) $(RMDIR) "$(DATA_DIR)\tgt"
	$(IF_EXIST) "$(DATA_DIR)\Config" $(DO) $(RMDIR) "$(DATA_DIR)\Config"
	$(IF_EXIST) "$(DATA_DIR)\DE_EPAGES" $(DO) $(RMDIR) "$(DATA_DIR)\DE_EPAGES"
	$(DEL) "$(DATA_DIR)\Cartridges.xml"
	$(PRL) Scripts/DeleteOldStoreTypeDirs.pl -current $(PATCH_VERSION)
	$(IF_EXIST) "$(EPAGES_CARTRIDGES)\.version" $(DO) $(RMDIR) "$(EPAGES_CARTRIDGES)\.version"
	$(IF_EXIST) "$(EPAGES_CARTRIDGES)\.lang" $(DO) $(RMDIR) "$(EPAGES_CARTRIDGES)\.lang"
	$(DEL) $(EPAGES_CARTRIDGES)/.6.*
	$(DEL) $(EPAGES_SHARED)/Monitor/.6.*
	$(DEL) $(EPAGES_STORES)/.6.*
	$(DEL) $(EPAGES_WEBROOT)/.6.*
	$(DEL) $(EPAGES_CONFIG)/.6.*
	$(DEL) $(EPAGES)/Service/.6.*
	$(DEL) $(EPAGES)/RequestRouter/.6.*
	$(DEL) $(EPAGES)/WebAdapter/.6.*
	$(DEL) $(EPAGES_PERL)/.6.*
	$(DEL) $(EPAGES)/bin/.6.*

transform_text2string_attribute:
	$(PRL) _CARTRIDGES_/DE_EPAGES/Object/Scripts/transformTextattributeToStringAttributePatch_6_13_0.pl -store all

#-*- Makefile -*-
