#!/bin/env bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBOyliXl5jMabHSgyARAhJrAJ45LOff26yqDAGG8eJQJXQtcUNGDwCcDXI5
#vl3YDDqueRDnM+a9yf/Elw0=
#=fNbU
#-----END PGP SIGNATURE-----

BN=${0##*/}

usage() {
  ( echo "Usage:"
    echo "  $BN [-b base] [-u update] [-s setup] [-r repository]"
    echo "Options:"
    echo "  -b: base version for RPMs as e.g. 6.12"
    echo "  -u: update version for RPMs as e.g. 6.12"
    echo "  -s: setup version for RPMs as e.g. 6.12"
    echo "  -r: get RPMs from repository" ) 1>&2
  exit 1
}

EPAGES_RPMS_URL=${EPAGES_RPMS_URL:-ftp://epages-software.de/pub/products/RPMS}
unset BASE_VER UPDATE_VER SETUP_VER DEV_VER
while getopts b:u:s:r: c ; do
  case $c in
    b) BASE_VER=$OPTARG ; [[ -n ${BASE_VER//[0-9.]/} ]] && usage ;;
    u) UPDATE_VER=$OPTARG ; [[ -n ${UPDATE_VER//[0-9.]/} ]] && usage ;;
    s) SETUP_VER=$OPTARG ; [[ -n ${SETUP_VER//[0-9.]/} ]] && usage ;;
    r) EPAGES_RPMS_URL=$OPTARG ;;
    *) usage ;;
  esac
done

case $EPAGES_RPMS_URL in
  */upload/6/*/[0-9]) CUT_DIRS=5 ;;
  */upload/6/*/[0-9]/6.*[0-9]) CUT_DIRS=5 ; DEV_VER=${EPAGES_RPMS_URL##*/}
    EPAGES_RPMS_URL=${EPAGES_RPMS_URL%/*} ;;
  ftp://epages-software.de/pub/products/RPMS) CUT_DIRS=3 ;;
  *) echo "Error: $BN -r $EPAGES_RPMS_URL: repository not supported" ; exit 1 ;;
esac

umask 022
export LANG=C

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

rpm -q wget || exit 1

NEW_VER=$(rpm -qp "$EPAGES_RPMS_URL/epages-version.noarch.rpm" --qf '[%{VERSION}\n]' 2>/dev/null | egrep '^[0-9.]+$')

if [[ -z $NEW_VER ]] ; then
  echo -e "\nCannot get ePages RPM: $EPAGES_RPMS_URL/epages-version.noarch.rpm"
  echo -e "Try running this command on an internet connected host.\n"
  exit 1
fi

BASE_VER=${BASE_VER:-${NEW_VER%.*}}
UPDATE_VER=${UPDATE_VER:-$BASE_VER}-update
EMPTY_VER=${UPDATE_VER%-update}-empty
SETUP_VER=${SETUP_VER:-$BASE_VER}-setup
BASE_VER=${DEV_VER:-$BASE_VER}

# -----------------------------------------------------------------------------
# find epages environment
# -----------------------------------------------------------------------------

if [[ -z $EPAGES_PROJECT && -f /etc/sysconfig/epages6 ]] ; then
   . /etc/sysconfig/epages6
  EPAGES_PROJECT=$EPAGES_INSTALL_RUN_SCRIPT
fi

EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

if [[ -f /etc/default/$EPAGES_PROJECT ]] ; then
  . /etc/default/$EPAGES_PROJECT
elif rpm --quiet -q epages-common ; then
  EPAGES_PARENT="$(rpm -q epages-common --qf '[%{INSTPREFIXES}\n]' | head -1)"
  EPAGES_SHARED="$EPAGES_PARENT"/eproot/Shared
else
  EPAGES_SHARED=/srv/epages/eproot/Shared
fi

# -----------------------------------------------------------------------------
# download repository
# -----------------------------------------------------------------------------

mkdir -m 755 -p "$EPAGES_SHARED"/RPMS
chown ep_appl:epages $(dirname "$EPAGES_SHARED") "$EPAGES_SHARED"

cd "$EPAGES_SHARED"/RPMS || exit 1
echo pwd: `pwd`

for i in epages-release.noarch.rpm epages-version.noarch.rpm kernel rpmify\
 $BASE_VER $UPDATE_VER $EMPTY_VER $SETUP_VER ; do
  echo "wget -np -nv -nH --cut-dirs=$CUT_DIRS --mirror $EPAGES_RPMS_URL/$i"
  LD_LIBRARY_PATH= wget -np -nv -nH --cut-dirs=$CUT_DIRS --mirror $EPAGES_RPMS_URL/$i
done
