#!/bin/env bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBOyliXl5jMabHSgyARAvAPAJ9fgo2VnnIEEVmWQvv+8rLBEZBIMgCeN02i
#BfApIY8oxhsRjWCG+5/kfWk=
#=nj+N
#-----END PGP SIGNATURE-----

[ -n "$EPAGES_IGNORE_REWRITE_CONF" ] && exit 0

BN=`basename $0`

[ "x-show" = "x$1" ] && SHOW_REWRITE=1 || unset SHOW_REWRITE

if [ $# -gt 0 -a "x-show" != "x$1" ] ; then
  echo "Usage:"
  echo "  [EPAGES_CLOSE_WEBSITE=1] [WEBSRV_PROCESS_NAME=<MY_NAME>] $0 [-show]"
  echo "Example:"
  echo "  WEBSRV_PROCESS_NAME=httpd-worker \$EPAGES/bin/$BN -show"
  exit 1
fi

# exit if ePages environment incomplete
if [ -z "$EPAGES" -o -z "$EPAGES_CONFIG" ] ; then
  EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}
  . /etc/default/"$EPAGES_PROJECT"
fi
if [ -z "$EPAGES" -o -z "$EPAGES_CONFIG" -o -z "$EPAGES_REWRITE_CONF" ] ; then
  if [ -n "$SHOW_REWRITE" ] ; then
    echo "$BN: ePages environment incomplete in /etc/default/\$EPAGES_PROJECT:"
    echo "  Correct the values of following environment variables:"
    echo "  EPAGES_PROJECT=$EPAGES_PROJECT"
    echo "  EPAGES=$EPAGES"
    echo "  EPAGES_CONFIG=$EPAGES_CONFIG"
    echo "  EPAGES_REWRITE_CONF=$EPAGES_REWRITE_CONF"
  fi
  exit 0
fi

# exit if directory doesn't exist
if [ ! -d "$(dirname "$EPAGES_REWRITE_CONF")" ] ; then
  if [ -n "$SHOW_REWRITE" ] ; then
    echo "$BN: Directory for EPAGES_REWRITE_CONF must exist:"
    echo "$(dirname "$EPAGES_REWRITE_CONF"): No such file or directory"
  fi
  exit 0
fi

# create $EPAGES_REWRITE_CONF unless exists
if [ ! -f "$EPAGES_REWRITE_CONF" ] ; then
  if ! touch "$EPAGES_REWRITE_CONF" ; then
    [ -n "$SHOW_REWRITE" ] && echo "$BN: Can't write \$EPAGES_REWRITE_CONF"
    exit 0
  fi
  if [ -w /etc/passwd ] ; then
    OWNER=${EPAGES_WEBGROUP##*:}
    [ -z "$OWNER" ] && OWNER=ep_web || OWNER=ep_web:$OWNER
    [ -n "$OWNER" ] && chown $OWNER "$EPAGES_REWRITE_CONF"
  fi
fi

# exit if file ini-config.sh isn't executable
PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin
type ini-config.sh >/dev/null 2>&1 || {
  [ -n "$SHOW_REWRITE" ] && echo "$BN: Can't execute ini-config.sh"
  exit 1
}

# exit if file WebInterface.conf isn't readable
if [ ! -r "$EPAGES_CONFIG/WebInterface.conf" ] ; then
  [ -n "$SHOW_REWRITE" ] && echo "$BN: Can't read \$EPAGES_CONFIG/WebInterface.conf"
  exit 1
fi

# SHOW_REWRITE
if [ -n "$SHOW_REWRITE" ] ; then
  # exit if perl doesn't work
  $PERL -MLWP::Simple -e 1 || exit 1

  # exit if web server is not running, else set WEBSRV_PID
  if [ -n "$WEBSRV_PROCESS_NAME" ] ; then
    set -- `pgrep -u root $WEBSRV_PROCESS_NAME`
  else
    set -- `pgrep -u root httpd-epages`
    [ $# -eq 0 ] && set -- `pgrep -u root httpd`
  fi
  if [ $# -eq 0 ] ; then
    echo "$BN: ${WEBSRV_PROCESS_NAME:-httpd}: no such process found"
    echo "  Make sure that a) the web server is running and"
    echo "  b) the name of the web server (${WEBSRV_PROCESS_NAME:-httpd}) is correct."
    echo "  If the name of the web server differs, run:"
    echo "    WEBSRV_PROCESS_NAME=<MY_NAME> \$EPAGES/bin/$BN -show"
    echo "  Example:"
    echo "    WEBSRV_PROCESS_NAME=httpd-worker \$EPAGES/bin/$BN -show"
    exit 1
  elif [ $# -eq 1 ] ; then
    WEBSRV_PID=$1
    ps -p${WEBSRV_PID// /,} -ouid_hack,pid,stime,args
  else
    echo "Various web server processes found:"
    WEBSRV_PID=$@
    i=CHOOSE ; ps -p${WEBSRV_PID// /,} -ouid_hack,pid,stime,args | while read line ; do
      echo "$line [$i]"; i=`expr $i + 1 2>/dev/null || echo 1`
    done
    echo -n "Choose the right web server process (1 .. $#): "
    declare -i ans ; read ans
    if [ $ans -lt 1 -o $ans -gt $# ] ; then
      echo "$BN: Aborted."
      exit 0
    fi
    WEBSRV_PID=`eval echo '$'$ans`
  fi
fi

# read configuration values
[ -n "$SHOW_REWRITE" ] && echo -e "\nread  conf: $EPAGES_CONFIG/WebInterface.conf"
RW_EPAGES=$(ini-config.sh -f "$EPAGES_CONFIG"/WebInterface.conf -s URLRewrite -k epages -g)
RW_BASEURI=$(ini-config.sh -f "$EPAGES_CONFIG"/WebInterface.conf -s URLRewrite -k BaseURI -g)
RW_EX_PREFIX=$(ini-config.sh -f "$EPAGES_CONFIG"/WebInterface.conf -s URLRewrite -k ExceptionPrefix -g)
RW_EX_REGEX=$(ini-config.sh -f "$EPAGES_CONFIG"/WebInterface.conf -s URLRewrite -k ExceptionRegex -g)
RW_EX_HOST=$(ini-config.sh -f "$EPAGES_CONFIG"/WebInterface.conf -s URLRewrite -k ExceptionHost -g)
LOCAL_HOSTS=$(ini-config.sh -f "$EPAGES_CONFIG"/WebInterface.conf -s URLRewrite -k LocalHosts -g)

#  write new $EPAGES_REWRITE_CONF or exit
[[ -n $EPAGES_CLOSE_WEBSITE && -z $LOCAL_HOSTS ]] && exit 0

if [ -z "$RW_EX_PREFIX$RW_EX_REGEX$RW_EX_HOST$EPAGES_CLOSE_WEBSITE" ] ; then
  [ -n "$SHOW_REWRITE" ] && echo "$BN: \$EPAGES_CONFIG/WebInterface.conf: No rewrite instruction found"
  cat /dev/null > "$EPAGES_REWRITE_CONF"
  exit 0
fi

echo -e '<IfModule rewrite_module>\nRewriteEngine on' > "$EPAGES_REWRITE_CONF".new || exit 1
[ -n "$SHOW_REWRITE" ] && echo -e 'RewriteLogLevel 2\nRewriteLog "/tmp/EPAGES_SHOW_REWRITE.log.~0815~"' >> "$EPAGES_REWRITE_CONF".new

RW_EPAGES=${RW_EPAGES:-/epages/}

if [ -n "$EPAGES_CLOSE_WEBSITE" ] ; then
  for i in localhost 127.0.0.1 ::1 $LOCAL_HOSTS ; do
    i="$(echo "$i" | sed 's/\([].[]\)/\\\1/'g)"
    echo 'RewriteCond %{REMOTE_HOST} !^'"$i"'$ [NC]' >> "$EPAGES_REWRITE_CONF".new
    echo 'RewriteCond %{REMOTE_ADDR} !^'"$i"'$ [NC]' >> "$EPAGES_REWRITE_CONF".new
  done

  cat <<EOF >> "$EPAGES_REWRITE_CONF".new
RewriteCond %{REQUEST_URI} !=/WebRoot/StoreClosed.html
RewriteRule ^ - [R=503,L]
</IfModule>

ErrorDocument 503 /WebRoot/StoreClosed.html
EOF

else
  for i in $RW_EX_HOST ; do
    i="$(echo "$i" | sed 's/\([].[]\)/\\\1/'g)"
    echo 'RewriteCond %{HTTP_HOST} !^'"$i"'$ [NC]' >> "$EPAGES_REWRITE_CONF".new
  done

  for i in $RW_EX_PREFIX ; do
    i="$(echo "$RW_BASEURI$i" | sed 's,///*,/,g')"
    echo 'RewriteCond %{REQUEST_URI} !^'"$i"'' >> "$EPAGES_REWRITE_CONF".new
  done
  for i in $RW_EX_REGEX ; do
    echo 'RewriteCond %{REQUEST_URI} !'"$i"'' >> "$EPAGES_REWRITE_CONF".new
  done

  echo 'RewriteRule ^'"${RW_BASEURI%/}"'/(.*)$ '"${RW_EPAGES%/}"'/.shorturl/?URI=$1 [PT,QSA]' >> "$EPAGES_REWRITE_CONF".new
  echo '</IfModule>' >> "$EPAGES_REWRITE_CONF".new
fi

# backup existing $EPAGES_REWRITE_CONF
if [ -n "$SHOW_REWRITE" ] ; then
  cp -p "$EPAGES_REWRITE_CONF" "$EPAGES_REWRITE_CONF".sofar
else
  cmp "$EPAGES_REWRITE_CONF" "$EPAGES_REWRITE_CONF".new >/dev/null 2>&1 \
  || cp -p "$EPAGES_REWRITE_CONF" "$EPAGES_REWRITE_CONF".~$(date +%s)~
fi
[ -n "$SHOW_REWRITE" ] && echo "write conf: $EPAGES_REWRITE_CONF"

# copy new $EPAGES_REWRITE_CONF
cat "$EPAGES_REWRITE_CONF".new > "$EPAGES_REWRITE_CONF" && rm -f "$EPAGES_REWRITE_CONF".new

# SHOW_REWRITE
if [ -n "$SHOW_REWRITE" -a -n "$WEBSRV_PID" ] ; then

  kill -HUP $WEBSRV_PID

  cat /dev/null > /tmp/EPAGES_SHOW_REWRITE.log.~0815~
  echo -e "\n$PERL -MLWP::Simple -e 'get \"http://127.0.0.1/TEST\" || get \"http://[::1]/TEST\"'"
  $PERL -MLWP::Simple -e 'get "http://127.0.0.1/TEST" || get "http://[::1]/TEST"'
  if awk -F\] '{sub(/- - /,"");sub(/\(2\) /,"");printf "%s]%s\n", $1, $4}' \
  < /tmp/EPAGES_SHOW_REWRITE.log.~0815~ && [ -w /etc/passwd  ] ; then
    GROUP=${EPAGES_WEBGROUP##*:}
    GROUP=${GROUP:-epages}
    chown ep_appl:$GROUP /tmp/EPAGES_SHOW_REWRITE.log.~0815~
    chmod 664 /tmp/EPAGES_SHOW_REWRITE.log.~0815~
  fi
  grep -q "/epages/\.shorturl/" /tmp/EPAGES_SHOW_REWRITE.log.~0815~ || {
    echo -e "\n$BN: ${WEBSRV_PROCESS_NAME:-httpd}: Rewrite failed."
  }
  mv "$EPAGES_REWRITE_CONF".sofar "$EPAGES_REWRITE_CONF"
  kill -HUP $WEBSRV_PID || {
    echo -e "\n$BN: Restart of web server failed.\nPlease start the web server by yourself."
  }
fi
