#!/bin/env bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBOzP0tl5jMabHSgyARAmbZAKCRbRVPTCVbEaFP2YOIl2JptVDLvgCgrZ/5
#BpIlVS3wdx/DrRgmza09cRg=
#=Sm9O
#-----END PGP SIGNATURE-----

BN=${0##*/}
EPOCH=`date "+%s"`

if [[ ! -w /etc/passwd ]] ; then
  echo "$BN: Only 'root' can execute this script."
  exit 1
fi

. /var/epages/functions.sh
PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin

WEB_SERVER=$(httpd_name)
WEB_SERVER=${WEB_SERVER:-httpd}

usage() {
  echo "Usage:"
  echo "  $BN [-g WEB_GROUP] [-w WEB_USER] [-u] [-r]"
  echo "Options:"
  echo "  -g: web server group"
  echo "  -w: web server user"
  echo "  -u: undo changes on local web server"
  echo "  -r: rewrite configuration in web server configuration directory"
  exit 1
}

unset WEB_GROUP WEB_USER UNDO REWRITE
while getopts g:w:ur c ; do
  case $c in
    g) WEB_GROUP=$OPTARG ;;
    w) WEB_USER=$OPTARG ;;
    u) UNDO=1 ;;
    r) REWRITE=1 ;;
    *) usage ;;
  esac
done

ETC_DEFAULT=$(etc_default $(cd $(dirname $0)/../.. && pwd))
ls "$ETC_DEFAULT" >/dev/null || exit 1
. "$ETC_DEFAULT"

. $EPAGES/bin/configure-epages.sh || exit 1

LOG_DIR=/var/log/$WEB_SERVER

[ -d $LOG_DIR ] || mkdir -m 750 -p $LOG_DIR 2>/dev/null
[ -d $LOG_DIR ] || { LOG_DIR=$HOME/log/$WEB_SERVER; mkdir -m 750 -p $LOG_DIR; }

CONF_DIR=/etc/$WEB_SERVER
CONF_D=$(httpd_confd)

RUN_SCRIPT=/etc/init.d/$WEB_SERVER
[[ $WEB_SERVER = httpd ]] && DEFAULT_GROUP=apache || DEFAULT_GROUP=www
WEB_GROUP=${WEB_GROUP:-$DEFAULT_GROUP}
[[ $WEB_SERVER = httpd ]] && DEFAULT_USER=apache || DEFAULT_USER=wwwrun
WEB_USER=${WEB_USER:-$DEFAULT_USER}

if [[ -f /usr/sbin/httpd2 ]] ; then
  HTTPD_EXE=/usr/sbin/httpd2
elif [[ -f /usr/sbin/httpd ]] ; then
  HTTPD_EXE=/usr/sbin/httpd
else
  HTTPD_EXE=`ls /usr/sbin/httpd2* 2>/dev/null | head -1`
  if [[ -n "$HTTPD_EXE" ]] ; then
    ln -s $HTTPD_EXE /usr/sbin/httpd2
  else
    HTTPD_EXE=`ls /usr/sbin/httpd* | head -1`
  fi
fi
[[ -z "$HTTPD_EXE" ]] && HTTPD_EXE=`which httpd`
[[ -z "$HTTPD_EXE" ]] && HTTPD_EXE=`which apache2`

#-----------------------------------------------------------------------
# reset configuration
#-----------------------------------------------------------------------

if [[ -n "$UNDO" ]] ; then

  if [[ -n "$CONF_DIR" ]] ; then
    # reset $CONF_D/*epages-*.conf
    echo "Undo: $CONF_DIR/**epages**"
    find "$CONF_DIR" -follow -type f -name "*epages*" -exec rm \-f {} \; 2>/dev/null

    # reset $CONF_DIR/httpd.conf
    for i in `find "$CONF_DIR" -type f -name httpd.conf ! -regex '.*/original/.*' 2>/dev/null` ; do
      if grep -q "epages" $i 2>/dev/null ; then
        ORIG=`ls $i.~*~ 2>/dev/null | head -1`
        echo "Undo: $i"
        [[ -n "$ORIG" ]] && cat "$ORIG" > $i
        rm -f $i.~*~
        cp -p $i $i.tmp && sed 's/#epages#//g' $i.tmp | grep -v epages > $i
      fi
      rm -f $i.tmp $i.~*~
    done
  fi

  # reset $EPAGES/Apache
  if [[ -d "$EPAGES"/Apache.unused ]] ; then
    rm -rf "$EPAGES"/Apache.new
    [[ -d "$EPAGES"/Apache ]] && mv "$EPAGES"/Apache "$EPAGES"/Apache.new
    mv "$EPAGES"/Apache.unused "$EPAGES"/Apache
    echo "Undo: $EPAGES/Apache"

    NEW_WEB_GROUP=`find "$EPAGES"/Apache -maxdepth 0 -follow -printf "%g\n"`
    if [[ "$NEW_WEB_GROUP" != "${EPAGES_WEBGROUP%%:*}" ]] ; then
      # change web group
      chgrp -L root / >/dev/null 2>&1 && CHGRP_L=-L || unset CHGRP_L
      chgrp -fR $CHGRP_L "${EPAGES_WEBGROUP%%:*}" "$EPAGES"/Apache/
    fi
  fi

  if [[ -n "$RUN_SCRIPT" ]] ; then
    # reset $RUN_SCRIPT
    if grep -q "/etc/default/.\?$EPAGES_PROJECT.\?$" $RUN_SCRIPT 2>/dev/null ; then
      echo "Undo: $RUN_SCRIPT"
      ORIG=`ls $RUN_SCRIPT.~*~ 2>/dev/null | head -1`
      [[ -n "$ORIG" ]] && cat "$ORIG" > $RUN_SCRIPT
      cp -p $RUN_SCRIPT $RUN_SCRIPT.tmp \
      && egrep -v "(/etc/default/.\?$EPAGES_PROJECT.\?$|update-rewrite-conf\.sh|^export (LD_LIBRARY_)\?PATH=|setsebool)" \
        $RUN_SCRIPT.tmp > $RUN_SCRIPT
    fi
    rm -f $RUN_SCRIPT.tmp $RUN_SCRIPT.~*~
    $RUN_SCRIPT stop >/dev/null 2>&1
    echo Starting web server ...
    sleep 2
    $RUN_SCRIPT start
    sleep 2
    ps -f -C ${HTTPD_EXE##*/}
  fi

  exit 0
fi

#-----------------------------------------------------------------------
# define environment
#-----------------------------------------------------------------------

ls "$CONF_D" >/dev/null || usage
ls "$RUN_SCRIPT" >/dev/null || exit 1
[[ -z "$HTTPD_EXE" ]] && { echo "Error: No httpd binary found.";  exit 1; }
ls "$EPAGES"/WebAdapter/zzz-epages-{ssl,httpd}.conf.ept >/dev/null || exit 1

WEB_GROUP_ID=$(getent group "$WEB_GROUP" | awk -F: '{print $3}')
[[ -z "$WEB_GROUP_ID" ]] && { echo "$BN: group not found: $WEB_GROUP"; exit 1; }

LD_LIBRARY_PATH=$(epages_libs | tr ':' '\n' | grep -v $EPAGES/Apache | tr '\n' ':')

$HTTPD_EXE -v 2>/dev/null | grep -q 'Apache/2\.2[^0-9]' || {
  echo "$BN: requires Apache 2.2"
  $HTTPD_EXE -v
  exit 1
}

file -L $HTTPD_EXE | grep -q 'ELF 64-bit' && WA_LIB=lib64/ || WA_LIB=

if [[ -f /etc/SuSE-release ]] ; then
  [[ -z $WA_LIB ]] && MODULES_PATH=/usr/lib/apache2* || MODULES_PATH=/usr/lib64/apache2*
else
  [[ -z $WA_LIB ]] && MODULES_PATH=/usr/lib/httpd/modules || MODULES_PATH=/usr/lib64/httpd/modules
fi

#-----------------------------------------------------------------------
# set configuration
#-----------------------------------------------------------------------

unset SERVER_ALIAS
if [[ $EPAGES_INSTALL_WS_ADDR != ${EPAGES_INSTALL_WS_FQDN%%.*} ]] ; then
  SERVER_ALIAS="ServerAlias ${EPAGES_INSTALL_WS_FQDN%%.*}"
fi
if [[ $EPAGES_INSTALL_WS_ADDR != $EPAGES_INSTALL_WS_FQDN && $EPAGES_INSTALL_WS_FQDN != ${EPAGES_INSTALL_WS_FQDN%%.*} ]] ; then
  SERVER_ALIAS="$SERVER_ALIAS\nServerAlias $EPAGES_INSTALL_WS_FQDN"
fi

# create dummy certificate
[[ -s "$CONF_DIR"/conf/ssl.crt/dummy-server.crt ]] && USE_SSL=1 || {
  echo "Run: $EPAGES/bin/dummycert.sh -h $EPAGES_INSTALL_WS_FQDN -c $CONF_DIR"
  "$EPAGES"/bin/dummycert.sh -h $EPAGES_INSTALL_WS_FQDN -c "$CONF_DIR" >/dev/null 2>&1
   [ -s "$CONF_DIR"/conf/ssl.crt/dummy-server.crt ] && USE_SSL=1 || unset USE_SSL
}

# edit $CONF_D/zzz-epages-lb.local
[[ -s "$CONF_D"/zzz-epages-lb.local && -z "$REWRITE" ]] || {
  echo "Edit: $CONF_D/zzz-epages-lb.local"
  sed "
  s, logs/, $LOG_DIR/,g;
  s,_CONF_DIR_,$CONF_DIR,g;
  s,_SERVERALIAS_,$SERVER_ALIAS,g;
  s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g;
  s,_IP_,$EPAGES_INSTALL_WS_IP,g;
  s,_REWRITE_CONF_,$EPAGES_REWRITE_CONF,g;
  " "$EPAGES"/WebAdapter/zzz-epages-lb.local > "$CONF_D"/zzz-epages-lb.local
}

# edit $CONF_D/zzz-epages-lb.conf.loop.ept
[[ -s "$CONF_D"/zzz-epages-lb.conf.loop && -z "$REWRITE" ]] || {
  echo "Edit: $CONF_D/zzz-epages-lb.conf.loop"
  sed "s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g" \
  "$EPAGES"/WebAdapter/zzz-epages-lb.conf.loop.ept >  "$CONF_D"/zzz-epages-lb.conf.loop
}

# edit $CONF_D/zzz-epages-ssl.conf
[[ -s "$CONF_D"/zzz-epages-ssl.conf && -z "$REWRITE" ]] || {
  echo "Edit: $CONF_D/zzz-epages-ssl.conf"
  sed "
  s, logs/, $LOG_DIR/,g;
  s,_CONF_DIR_,$CONF_DIR,g;
  s,_EPAGES_,$EPAGES,g;
  s,_SERVERALIAS_,$SERVER_ALIAS,g;
  s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g;
  s,_PORT_,$EPAGES_INSTALL_WS_PORT,g;
  s,_SSLPORT_,$EPAGES_INSTALL_WS_SSL_PORT,g;
  s,_LOCALPORT_,$EPAGES_INSTALL_WS_LOCAL_PORT,g;
  s,_LOCALSSLPORT_,$EPAGES_INSTALL_WS_LOCAL_SSLPORT,g;
  s,_IP_,$EPAGES_INSTALL_WS_IP,g;
  s,_REWRITE_CONF_,$EPAGES_REWRITE_CONF,g;
  " "$EPAGES"/WebAdapter/zzz-epages-ssl.conf.ept > "$CONF_D"/zzz-epages-ssl.conf
}

MODULES="authz_host deflate log_config env expires headers setenvif mime
autoindex cgi vhost_alias negotiation dir actions alias rewrite ssl"

FOUND_MODULES=

# edit $CONF_D/zzz-epages-httpd.conf
[[ -s "$CONF_D"/zzz-epages-httpd.conf ]] || {
  echo "Edit: $CONF_D/zzz-epages-httpd.conf"
  [ "$(/bin/uname -p)" = x86_64 ] && LIB64=lib64/ || unset LIB64

  sed "
  s,_EPAGES_PERL_,$EPAGES_PERL,g;
  s,_EPAGES_,$EPAGES,g;
  s,_HOST_,$EPAGES_INSTALL_WS_ADDR,g;
  s,_PORT_,$EPAGES_INSTALL_WS_PORT,g;
  s,_LOCALPORT_,$EPAGES_INSTALL_WS_LOCAL_PORT,g;
  s,_IP_,$EPAGES_INSTALL_WS_IP,g;
  s,_KERNEL_,$EPAGES_KERNEL,g;
  s,_LIB_,$LIB64,g;
  " "$EPAGES"/WebAdapter/zzz-epages-httpd.conf.ept > "$CONF_D"/zzz-epages-httpd.conf

  for i in $MODULES ; do
    MODULE=$(find $MODULES_PATH -follow -type f -name "mod_$i.so" 2>/dev/null | head -1)
    if [[ -n "$MODULE" ]] ; then
      FOUND_MODULES="$FOUND_MODULES $i"
      sed "s, modules/mod_$i\.so, $MODULE," -i "$CONF_D"/zzz-epages-httpd.conf
    else
      sed "s,\(.* modules/mod_$i\.so\),#\1," -i "$CONF_D"/zzz-epages-httpd.conf
    fi
  done
}

# edit $CONF_D/ssl.conf
if [[ $WEB_SERVER = httpd ]] && grep -q '_default_:443' "$CONF_D"/ssl.conf 2>/dev/null ; then
  echo "Edit: $CONF_D/ssl.conf"
  sed -i 's,_default_:443,127.0.0.1:443,' "$CONF_D"/ssl.conf
fi

# edit /etc/sysconfig/$WEB_SERVER
if [[ -f /etc/sysconfig/$WEB_SERVER ]] ; then
  echo "Edit: /etc/sysconfig/$WEB_SERVER"
  ( SYS_SCRIPT=/etc/sysconfig/$WEB_SERVER
    . $SYS_SCRIPT
    if [[ -n ${!APACHE_MODULES*} ]] ; then
      am=" $APACHE_MODULES "
      for i in $FOUND_MODULES ; do
        [[ "${am/ $i /}" = "$am" ]] && APACHE_MODULES="$APACHE_MODULES $i"
      done
      if [[ $am != " $APACHE_MODULES " ]] ; then
        cp -p $SYS_SCRIPT $SYS_SCRIPT.tmp \
        && sed 's,^\([[:space:]]*APACHE_MODULES=\).*,\1"'"$APACHE_MODULES"'",' $SYS_SCRIPT.tmp > $SYS_SCRIPT \
        && rm -f $SYS_SCRIPT.tmp
      fi
    fi
    if [ -n "$USE_SSL" ] ; then
      if [[ $WEB_SERVER = apache2 && -z ${!APACHE_SERVER_FLAGS*} ]] ; then
        echo 'APACHE_SERVER_FLAGS="SSL"' >> $SYS_SCRIPT
      elif [[ $WEB_SERVER = apache2 && "${APACHE_SERVER_FLAGS/SSL/}" = "$APACHE_SERVER_FLAGS" ]] ; then
        cp -p $SYS_SCRIPT $SYS_SCRIPT.tmp \
        && sed 's,^\([[:space:]]*APACHE_SERVER_FLAGS=\).*,\1"'"$APACHE_SERVER_FLAGS SSL"'",' $SYS_SCRIPT.tmp > $SYS_SCRIPT \
        && rm -f $SYS_SCRIPT.tmp
      fi
      if [[ -z ${!OPTIONS*} ]] ; then
        echo OPTIONS=-DSSL >> $SYS_SCRIPT
      elif [[ "${OPTIONS/-DSSL/}" = "$OPTIONS" ]] ; then
        cp -p $SYS_SCRIPT $SYS_SCRIPT.tmp \
        && sed 's,^\([[:space:]]*OPTIONS=\).*,\1"'"$OPTIONS -DSSL"'",' $SYS_SCRIPT.tmp > $SYS_SCRIPT \
        && rm -f $SYS_SCRIPT.tmp
      fi
    fi
  )
fi

# edit $CONF_D/zzz-epages-rewrite.conf
echo "Edit: $EPAGES_REWRITE_CONF"
"$EPAGES"/bin/update-rewrite-conf.sh

chmod 644 $CONF_D/zzz-epages*

# edit $CONF_DIR/httpd.conf
for i in `find "$CONF_DIR" -type f -name httpd.conf ! -regex '.*/original/.*'` ; do
  grep -q '#epages#' "$i" && continue
  echo "Edit: $i"
  cp -p "$i" "$i".~$EPOCH~
  ( sed 's,^\([[:space:]]*DocumentRoot[[:space:]]\),#epages#\1,' $i.~$EPOCH~
    echo -e '<IfModule !FCGIAdapter_mod>\nInclude '"$CONF_D"'/zzz-epages-*.conf\n</IfModule>'
  ) > $i
done

# edit DocumentRoot in $CONF_DIR
for i in `find "$CONF_DIR" -type f -name '*.conf' ! -regex '.*/original/.*' ` ; do
  rm -f $i.tmp
  cp -p "$i" "$i".~$EPOCH~
  awk '/^[[:space:]]*DocumentRoot/{print $2}' < $i | while read j ; do
    eval e=$j
    if [[ ! -d "$j" && ! -d "$e" ]] ; then
      echo "Edit: $i"
      cp -p "$i" "$i".tmp && grep -v "DocumentRoot[[:space:]].*$j" $i.tmp > $i
    fi
  done
  [[ -e "$i".tmp ]] && rm -f "$i".tmp || rm -f "$i".~$EPOCH~
done

# edit /etc/default/$EPAGES_PROJECT
cp -p "$FILE" "$FILE".~$EPOCH~
echo "Edit: $FILE"
set -- `getent group "$WEB_GROUP" | tr : ' '`
ini-config.sh -f "$FILE" -k EPAGES_WEBGROUP -v "$1:$3" -i
ini-config.sh -f "$FILE" -k EPAGES_WEBUSER -v $WEB_USER -i

# edit /etc/group
for i in ep_appl ${EPAGES_APPUSER:-eprunapp} ; do
  MEMBERS=$(awk -F: '{if ($1 == "'$WEB_GROUP'") {print $4; exit 0}}' /etc/group)
  MEMBERS=${MEMBERS:+$MEMBERS,}
  if [[ ",$MEMBERS" !=  *",$i,"* ]] ; then
    echo "Edit: /etc/group: $i"
    cp -p /etc/group /etc/group.~$EPOCH~
    sed "s/^\($WEB_GROUP:.*:\).*/\1$MEMBERS$i/" /etc/group.~$EPOCH~ > /etc/group
  fi
done

# edit $RUN_SCRIPT
grep -q "bin/update-rewrite-conf\.sh" $RUN_SCRIPT
if [[ $? -ne 0 ]] ; then
  echo "Edit: $RUN_SCRIPT"
  cp -p $RUN_SCRIPT $RUN_SCRIPT.~$EPOCH~
  grep -q '()' $RUN_SCRIPT && WS_PATTERN='()' || WS_PATTERN='^$'
  export WS_PATTERN
  ( sed -n "1,/$WS_PATTERN/p" $RUN_SCRIPT.~$EPOCH~ | grep -v "$WS_PATTERN"
    grep -q "/etc/default/.\?$EPAGES_PROJECT.\?$" $RUN_SCRIPT.~$EPOCH~ || {
      echo ". /etc/default/\"$EPAGES_PROJECT\""
      echo 'export LD_LIBRARY_PATH'
      echo 'export PATH=$PATH:$EPAGES_PERL/bin:/usr/bin:/usr/sbin:/sbin'
      echo 'if type -p setsebool >/dev/null 2>&1 ; then'
      echo '  setsebool httpd_can_network_connect=1 >/dev/null 2>&1'
      echo '  setsebool httpd_can_network_connect_db=1 >/dev/null 2>&1'
      echo '  setsebool httpd_can_network_relay=1 >/dev/null 2>&1'
      echo '  setsebool httpd_disable_trans=1 >/dev/null 2>&1'
      echo 'fi # setsebool'
    }
    echo '[ -x "$EPAGES"/bin/update-rewrite-conf.sh ] && "$EPAGES"/bin/update-rewrite-conf.sh'
    sed -n "/$WS_PATTERN/,"'$p' $RUN_SCRIPT.~$EPOCH~
  ) > $RUN_SCRIPT
fi

# stop epages web server
/etc/init.d/"$EPAGES_PROJECT" stop_httpd

OLD_WEB_GROUP=$EPAGES_WEBGROUP

. "$FILE"

if [[ "$OLD_WEB_GROUP" != "$EPAGES_WEBGROUP" ]] ; then
  # change web group
  chgrp -L root / >/dev/null 2>&1 && CHGRP_L=-L || unset CHGRP_L
  chgrp -fR $CHGRP_L $WEB_GROUP "$EPAGES_LOG/" "$EPAGES_WEBROOT/" "$EPAGES_SHARED"/Monitor/ "$EPAGES"/WebAdapter/
fi

# restart web server
if [[ -d "$EPAGES"/Apache ]] ; then
  rm -rf "$EPAGES"/Apache.unused
  mv "$EPAGES"/Apache "$EPAGES"/Apache.unused
fi

if [[ $RUN_SCRIPT != ${RUN_SCRIPT#/etc/init.d/} ]] ; then
  PERL5LIB= /sbin/chkconfig --del $WEB_SERVER >/dev/null 2>&1
  cp -p $RUN_SCRIPT $RUN_SCRIPT.tmp \
  && sed 's/\(chkconfig:\) -/\1 35/' $RUN_SCRIPT.tmp > $RUN_SCRIPT \
  && rm -f $RUN_SCRIPT.tmp
  chmod 755 $RUN_SCRIPT
  PERL5LIB= /sbin/chkconfig --add $WEB_SERVER >/dev/null 2>&1
else
  echo "Edit: /etc/init.d/httpd-epages"
  rm -f /etc/init.d/httpd-epages
  ln -s $RUN_SCRIPT /etc/init.d/httpd-epages
fi
$RUN_SCRIPT stop >/dev/null 2>&1
echo Starting web server ...
sleep 2
$RUN_SCRIPT start
sleep 2
ps -f -C ${HTTPD_EXE##*/}
exit 0
