#!/bin/bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBPUMFAl5jMabHSgyARAqGTAKCnxBsSlvsnqKHnDBdjVj4l50tqmQCfUuDt
#IPwtowHgc7FnHytT2rFvibg=
#=xSXB
#-----END PGP SIGNATURE-----

BN=${0##*/}

umask 022
export LANG=C

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

# -----------------------------------------------------------------------------
# find epages environment
# -----------------------------------------------------------------------------

VARIABLES_SOURCE=environment

if [[ -z $EPAGES_PROJECT && -f /etc/sysconfig/epages6 ]] ; then
   . /etc/sysconfig/epages6
  VARIABLES_SOURCE=sysconfig
  EPAGES_PROJECT=$EPAGES_INSTALL_RUN_SCRIPT
fi

if [[ -z $EPAGES_PROJECT ]] ; then
  VARIABLES_SOURCE=default
  EPAGES_PROJECT=epages6
fi

unset COMMON_VER COMMON_RPM_INSTALLED
if rpm --quiet -q epages-common ; then
  COMMON_RPM_INSTALLED=1
  EPAGES_PARENT="$(rpm -q epages-common --qf '[%{VERSION} %{INSTPREFIXES}\n]' | head -1)"
  X="${EPAGES_PARENT%%.*}" ; Y="${EPAGES_PARENT%% *}" ; Y=${Y#*.} ; Y=${Y%%.*}
  COMMON_VER=$X.$Y
  EPAGES_PARENT="${EPAGES_PARENT#* }"
fi

EPAGES_INSTALLED=1
if [[ -f /etc/default/$EPAGES_PROJECT ]] ; then
  . /etc/default/$EPAGES_PROJECT
else
 EPAGES="${EPAGES_PARENT:-/srv/epages}"/eproot
 EPAGES_SHARED="$EPAGES"/Shared
 [[ -n $COMMON_VER ]] && EPAGES_CONFIG="$EPAGES_SHARED"/Config || unset EPAGES_INSTALLED
fi

# -----------------------------------------------------------------------------
# evaluate command line
# -----------------------------------------------------------------------------

unset OWN_REPO DEV_REPO NEW_VER LOCAL_REPO DEVEL_VER

if [[ -z $EPAGES_INSTALL_VERSION ]] ; then
  EPAGES_RPMS_URL=ftp://epages-software.de/pub/products/RPMS
  RELEASE_RPM="$EPAGES_RPMS_URL/epages-release.noarch.rpm"
else
  EPAGES_RPMS_URL=http://www.epages.com/archive/RPMS/$EPAGES_INSTALL_VERSION
  NEW_VER=$EPAGES_INSTALL_VERSION
  RELEASE_RPM=http://www.epages.com/RPMS/epages-release.noarch.rpm
fi

if [[ $# -gt 0 && $1 != -repo ]] ; then
  echo "Usage: ${0##*/} [-repo REPOSITORY]"
  exit 1
fi

if [[ $1 = -repo && -n $2 ]] ; then
  EPAGES_RPMS_URL=$2
  OWN_REPO=" $1 $2"
  case $EPAGES_RPMS_URL in
    http://*/RPMS/6.*.[0-9])
      DEV_REPO=${EPAGES_RPMS_URL##*/} ; EPAGES_RPMS_URL=${EPAGES_RPMS_URL%/*}
      NEW_VER=$DEV_REPO ;;
    http://*/yum/6.*.[0-9]/6.*.[0-9])
      DEV_REPO=${EPAGES_RPMS_URL##*/} ; EPAGES_RPMS_URL=${EPAGES_RPMS_URL%/*}
      NEW_VER=${EPAGES_RPMS_URL##*/} ; DEVEL_VER=1 ;;
    http://*/yum/6.*.[0-9])
      NEW_VER=${EPAGES_RPMS_URL##*/} ;;
    */upload/6/*/[0-9]/6.*.[0-9])
      DEV_REPO=${EPAGES_RPMS_URL##*/} ; EPAGES_RPMS_URL=${EPAGES_RPMS_URL%/*}
      NEW_VER=${EPAGES_RPMS_URL#*/pub/upload/} ; NEW_VER=${NEW_VER//\//.} ; DEVEL_VER=1 ;;
    */upload/6/*/[0-9])
      NEW_VER=${EPAGES_RPMS_URL#*/pub/upload/} ; NEW_VER=${NEW_VER//\//.};;
    *://*/RPMS) true ;;
    /*/RPMS/6.*.[0-9])
      DEV_REPO=${EPAGES_RPMS_URL##*/} ; EPAGES_RPMS_URL=file://${EPAGES_RPMS_URL%/*}
      NEW_VER=$DEV_REPO ; DEVEL_VER=1 ;;
    /*/RPMS) EPAGES_RPMS_URL=file://$EPAGES_RPMS_URL ;;
    local/6.*.[0-9])
      DEV_REPO=${EPAGES_RPMS_URL##*/} ; EPAGES_RPMS_URL=file://$EPAGES_SHARED/RPMS
      NEW_VER=$DEV_REPO ; LOCAL_REPO=1 ; DEVEL_VER=1 ;;
    local)  EPAGES_RPMS_URL=file://$EPAGES_SHARED/RPMS ; LOCAL_REPO=1 ;;
    *) echo "Error: $0$OWN_REPO: repository not supported" ; exit 1 ;;
  esac
  shift 2
fi

# -----------------------------------------------------------------------------
# find current version
# -----------------------------------------------------------------------------

unset CURRENT_VER
if [[ -n $EPAGES_INSTALLED ]] ; then

  if ls $EPAGES_CONFIG/Cartridges.d/*.conf >/dev/null 2>&1 ; then
    CONF=$EPAGES_CONFIG/Cartridges.d/\*.conf
    CONFFILE=$EPAGES_CONFIG/Cartridges.d/*.conf
  else
    echo "Error: Install failed, file(s) does not exist:"
    echo "  \$EPAGES_CONFIG/Cartridges.d/*.conf"
    exit 1
  fi

  CURRENT_VER=$(awk -F= '/^DE_EPAGES::Core=/{print $2;exit 0}' $CONFFILE)
  if [[ -z $CURRENT_VER ]] ; then
    echo "Error: Install failed: Can't find DE_EPAGES::Core= in '$CONFFILE'."
    exit 1
  fi
fi

if [[ -z $NEW_VER ]] ; then
  NEW_VER=$(rpm -qp "$EPAGES_RPMS_URL/epages-version.noarch.rpm" --qf '[%{VERSION}\n]' 2>/dev/null | egrep '^[0-9.]+$')
fi

if [[ -z $NEW_VER && -z $OWN_REPO && -f "$EPAGES_SHARED/RPMS/epages-version.noarch.rpm" ]] ; then
  EPAGES_RPMS_URL="$EPAGES_SHARED/RPMS/epages-version.noarch.rpm"
  NEW_VER=$(rpm -qp "$EPAGES_RPMS_URL/epages-version.noarch.rpm" --qf '[%{VERSION}\n]' 2>/dev/null | egrep '^[0-9.]+$')
fi

unset UPDATE_VER
CURRENT_OR_NEW=${CURRENT_VER:-$NEW_VER}

if [[ -n "$CURRENT_OR_NEW" ]] ; then
  while IFS=. read -a i ; do REL=(${i[@]}) ; done <<<$CURRENT_OR_NEW

  [[ -n $EPAGES_INSTALLED ]] && MSG="$CONF: DE_EPAGES::Core=" || MSG="repository $EPAGES_RPMS_URL: "

  if [[ ${REL[0]} -ne 6 || ${REL[1]} -lt 9 || -z ${REL[2]} ]] ; then
    echo "Error: Install failed: $MSG$CURRENT_OR_NEW: void version"
    exit 1
  fi

  X=${REL[0]} ; Y=${REL[1]} ; Z=${REL[2]}
  UPDATE_VER=$X.$Y
fi

SETUP_VER=${COMMON_VER:-$UPDATE_VER}
BASE_VER=${DEV_REPO:-$SETUP_VER}

# -----------------------------------------------------------------------------
# exit if connection to repository fails
# -----------------------------------------------------------------------------

if [[ -z $NEW_VER ]] ; then

  [[ -n $BASE_VER ]] && ARGUMENTS=" -b $BASE_VER" || ARGUMENTS=
  [[ -n $UPDATE_VER && $UPDATE_VER != $BASE_VER ]] && ARGUMENTS="$ARGUMENTS -u $UPDATE_VER"
  [[ -n $SETUP_VER && $SETUP_VER != $BASE_VER ]] && ARGUMENTS="$ARGUMENTS -s $SETUP_VER"
  [[ -n $DEV_REPO && $DEV_REPO != $SETUP_VER ]] && REPO=/$DEV_REPO || REPO=

  echo -e "\nCannot connect to ePages repository: $EPAGES_RPMS_URL"
  echo "You may install a local repository by running following commands"
  echo -e "as root user on an internet connected host:\n"
  echo "  wget http://www.epages.de/RPMS/rpmify/install-repository.sh"
  echo "  chmod 755 install-repository.sh"
  echo "  ./install-repository.sh$ARGUMENTS"
  echo -e "\nThen move directory /srv/epages/eproot/Shared/RPMS"
  echo -e "to \$EPAGES_SHARED on your application server and re-run:\n"
  echo -e "  $0 -repo local$REPO $@\n"
  exit 1
fi

# -----------------------------------------------------------------------------
# check kernel version
# -----------------------------------------------------------------------------

[[ $(uname -p) = x86_64 ]] && ARCH=x86_64 || ARCH=i386

case $(uname -r) in
  2.6.16[.-]*) KERNEL=2.6.16s ;;
  2.6.18[.-]*) KERNEL=2.6.18r ;;
  2.6.27[.-]*) KERNEL=2.6.27s ;;
  2.6.32-*)    KERNEL=2.6.32r ;;
  2.6.32.*)    KERNEL=2.6.32s ;;
  *) echo "Error: Install failed: `uname -r`: Linux kernel version not supported."
     exit 1 ;;
esac

# -----------------------------------------------------------------------------
# check database type
# -----------------------------------------------------------------------------

cat $EPAGES_CONFIG/Database.conf $EPAGES_CONFIG/Database.d/*.conf 2>/dev/null \
| grep -q '^[[:space:]]*datasource[[:space:]]*=[[:space:]]*dbi:mysql' \
&& MYSQL_CLIENT=1 || unset MYSQL_CLIENT

unset DB_CLIENT_RPMS SERVER_RPMS

if [[ -n $MYSQL_CLIENT ]] ; then
  MY_IPS=$(LANG=C ifconfig -a | awk '/addr:/{sub(/.*addr: */,"",$0);sub(/[/ ].*/,"",$0);print}')
  for file in $(ls $EPAGES_CONFIG/Database.conf $EPAGES_CONFIG/Database.d/*.conf 2>/dev/null) ; do
    datasource=$(grep '^[[:space:]]*datasource[[:space:]]*=[[:space:]]*dbi:mysql' $file \
                 | sed 's/.*datasource[[:space:]]*=[[:space:]]*//;s/["''"]//g;' | head -1)
    [[ -z $datasource ]] && continue
    host=${datasource##*hostname=}
    host=${host##*host=}
    if [[ $host != $datasource ]] ; then
      host=${host%%;*}
      if [[ ${host//:/} = $host && -n ${host//[0-9.]/} ]] ; then
        host=$(dig +short $host +search +noall +answer | head -1)
        [[ -z $host ]] && continue
      fi
      for i in $MY_IPS ; do
        [[ $i = $host ]] && { SERVER_RPMS=mysqld ; break; }
      done
    else
      MY_CNF=$(grep '^[[:space:]]*MySQLConfigFile[[:space:]]*=' $file \
               | sed 's/.*MySQLConfigFile[[:space:]]*=[[:space:]]*//;s/["''"]//g;' | head -1)
      [[ -f ${MY_CNF:-/etc/my.cnf} ]] && { SERVER_RPMS=mysqld ; break; }
    fi
  done
fi

if [[ -n $MYSQL_CLIENT && ( -x $EPSERVICE || $SERVER_RPMS = mysqld ) ]] ; then
  DB_CLIENT_RPMS="$DB_CLIENT_RPMS mysql"
fi

# -----------------------------------------------------------------------------
# check installed servers
# -----------------------------------------------------------------------------

find /etc/{apache2,httpd} -name zzz-epages-httpd.conf 2>/dev/null \
| grep -q  . && SERVER_RPMS="$SERVER_RPMS webconf"

[[ -x $EPSERVICE ]] && SERVER_RPMS="$SERVER_RPMS appsrv"
[[ -x $EP_RR ]] && SERVER_RPMS="$SERVER_RPMS reqsrv"
[[ -x $EP_HTTPD ]] && SERVER_RPMS="$SERVER_RPMS websrv"

# -----------------------------------------------------------------------------
# create symlink /srv/epages if necessary
# -----------------------------------------------------------------------------

EPAGES_PARENT=/srv/epages

if [[ -n $EPAGES_INSTALLED ]] ; then
  [[ -d $EPAGES ]] && EPAGES_PARENT=$(cd $EPAGES/.. ; pwd) || EPAGES_PARENT=$(dirname $EPAGES)

  if [[ -d $EPAGES_PARENT/ && $(find $EPAGES/bin -maxdepth 0 -printf "%i") != $(find /srv/epages/eproot/bin -maxdepth 0 -printf "%i" 2>/dev/null) ]] ; then
    DATE=.~$(date +%s)~
    rm -f /srv/epages/{docs,etc,eproot} /srv/epages 2>/dev/null
    for i in $(find /srv/epages/{*,.[^.]*} -maxdepth 0 -printf "%f " 2>/dev/null) ; do
      ][ -f $(readlink -f $EPAGES_PARENT/$i) ]] || rm -f $EPAGES_PARENT/$i 2>/dev/null
      [[ -e $EPAGES_PARENT/$i ]] && mv /srv/epages/$i $EPAGES_PARENT/$i$DATE
      [[ -e $EPAGES_PARENT/$i ]] || mv /srv/epages/$i $EPAGES_PARENT/
    done
    rmdir /srv/epages 2>/dev/null
    rm -f /srv/epages 2>/dev/null
    mv /srv/epages /srv/epages$DATE 2>/dev/null
    mkdir -m 755 /srv
    ( cd /srv ; ln -s $EPAGES_PARENT epages )
  fi
fi

# -----------------------------------------------------------------------------
# find RPMS to install
# -----------------------------------------------------------------------------

echo "Using repository: $EPAGES_RPMS_URL"
echo "Using new version: $NEW_VER"
echo "Using install prefix: $EPAGES_PARENT"
echo "Using project name: $EPAGES_PROJECT"

[[ $VARIABLES_SOURCE = sysconfig ]] && echo "  Using sysconfig variables: /etc/sysconfig/epages6"
[[ -f /etc/default/$EPAGES_PROJECT ]] && echo "Using default variables: /etc/default/$EPAGES_PROJECT"

USED_VERSION=$SETUP_VER
if [[ -n $LOCAL_REPO ]] ; then
  USED_VERSION=${BASE_VER//./}
  [[ $(( ${#BASE_VER} - ${#USED_VERSION} )) -eq 2 ]] && USED_VERSION=${BASE_VER%.*} || USED_VERSION=$BASE_VER
  UPDATE_VER=$USED_VERSION ; SETUP_VER=$USED_VERSION
fi

[[ $UPDATE_VER != $SETUP_VER ]] && USED_VERSION="$USED_VERSION/$UPDATE_VER"
[[ -n $DEV_REPO && $DEV_REPO != $SETUP_VER ]] && USED_VERSION="$USED_VERSION/$DEV_REPO"
echo "Using repo version: $USED_VERSION"

unset REQ INSTALL_RPM SUSE_REQ INSTALLPREFIX
PATCH_VER=$(rpm -q epages-patch --qf '[%{VERSION}\n]' 2>/dev/null | egrep '^[0-9.]+$' | head -1)

# for suse add all requirements unless epages is installed
if [[ -z $EPAGES_INSTALLED && ${KERNEL%s} != $KERNEL ]] ; then
  DB_CLIENT_RPMS=mysql
  SERVER_RPMS="appsrv reqsrv mysqld webconf"
fi

[[ $Z = 0 ]] && MAJOR_PATCH= || MAJOR_PATCH=patch_${X}_${Y}_0
unset INSTALL_RELEASE_RPM
for i in release requires common $MAJOR_PATCH patch_${X}_${Y}_${Z} patch perl $DB_CLIENT_RPMS $SERVER_RPMS ; do

  [[ $i = patch_6_10_0 ]] && continue

  if rpm --quiet -q epages-$i ; then
    RPM_VER=$(rpm -q epages-$i --qf '[%{VERSION}\n]' | head -1)

    if [[ ${KERNEL%s} != $KERNEL && $PATCH_VER != ${NEW_VER:-$CURRENT_VER} && $RPM_VER != ${NEW_VER:-$CURRENT_VER} ]] ; then
      case $i in
        release|patch*) true ;;
        requires) SUSE_REQ="$SUSE_REQ $EPAGES_RPMS_URL/kernel/$KERNEL/epages-$i-$KERNEL-0.i386.rpm" ;;
        webconf) SUSE_REQ="$SUSE_REQ $EPAGES_RPMS_URL/rpmify/kernel/epages-$i-$KERNEL-0.i386.rpm" ;;
        *) SUSE_REQ="$SUSE_REQ $EPAGES_RPMS_URL/rpmify/epages-$i/epages-$i-${NEW_VER:-$CURRENT_VER}-0.i386.rpm" ;;
      esac
    fi
  else
    case $i in
      release) INSTALL_RELEASE_RPM=$RELEASE_RPM ;;
      requires) REQ="$REQ $EPAGES_RPMS_URL/kernel/$KERNEL/epages-$i-$KERNEL-0.i386.rpm" ;;
      webconf) REQ="$REQ $EPAGES_RPMS_URL/rpmify/kernel/epages-$i-$KERNEL-0.i386.rpm" ;;
      patch_${X}_${Y}_0) REQ="$REQ $EPAGES_RPMS_URL/rpmify/epages-patch/epages-$i-${CURRENT_OR_NEW%.*}.0-0.i386.rpm" ;;
      patch_${X}_${Y}_${Z}) REQ="$REQ $EPAGES_RPMS_URL/rpmify/epages-patch/epages-$i-$CURRENT_OR_NEW-0.i386.rpm" ;;
      *) REQ="$REQ $EPAGES_RPMS_URL/rpmify/epages-$i/epages-$i-${CURRENT_VER:-$NEW_VER}-0.i386.rpm" ;;
    esac
  fi
done

[[ -z $EPAGES_INSTALLED && ${KERNEL%s} != $KERNEL ]] && SUSE_REQ="$SUSE_REQ $REQ"

[[ -z $EPAGES_INSTALLED ]] && unset REQ

# -----------------------------------------------------------------------------
# install RPMs
# -----------------------------------------------------------------------------

if [[ -z $INSTALL_RELEASE_RPM ]] ; then
  for i in epages.repo epages-kernel.repo epages-setup.repo ; do
    [[ ! -s /etc/yum.repos.d/$i ]] && INSTALL_RELEASE_RPM=$RELEASE_RPM
  done
  [[ -n $INSTALL_RELEASE_RPM ]] && rpm -e epages-release
fi

if [[ -n $INSTALL_RELEASE_RPM ]] ; then
  rpm -hiv $@ $INSTALL_RELEASE_RPM 2>&1 | tee /tmp/${BN%.sh}.~$$~
fi
rpm --quiet -q epages-release && RELEASE_RPM_INSTALLED=1 || unset RELEASE_RPM_INSTALLED

PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin

if [[ ! -f /etc/yum.repos.d/epages-empty.repo ]] ; then
  sed "s/setup/empty/" /etc/yum.repos.d/epages-setup.repo > /etc/yum.repos.d/epages-empty.repo
  sed "s/setup/empty/" /etc/yum.repos.d/mirrors-epages-setup > /etc/yum.repos.d/mirrors-epages-empty
  chmod 644 /etc/yum.repos.d/*epages-empty*
  ini-config.sh -f /etc/yum.repos.d/epages-empty.repo -s epages-empty -k enabled -v 0 -i || exit 1
fi

if [[ -n $RELEASE_RPM_INSTALLED && ! -f /etc/yum.repos.d/epages-update.repo ]] ; then
  sed "s/setup/update/" /etc/yum.repos.d/epages-setup.repo > /etc/yum.repos.d/epages-update.repo
  sed "s/setup/update/" /etc/yum.repos.d/mirrors-epages-setup > /etc/yum.repos.d/mirrors-epages-update
  chmod 644 /etc/yum.repos.d/*epages-update*
fi

if [[ -n $RELEASE_RPM_INSTALLED ]] ; then
  if [[ -n $COMMON_RPM_INSTALLED ]] ; then
    ini-config.sh -f /etc/yum.repos.d/epages-setup.repo -s epages-setup -k enabled -v 0 -i || exit 1
    ini-config.sh -f /etc/yum.repos.d/epages-update.repo -s epages-update -k enabled -v 1 -i || exit 1
  fi
fi

if [[ -n $RELEASE_RPM_INSTALLED && -n $CURRENT_VER ]] ; then
  for repo in update empty ; do
    for i in $(ls -d /etc/yum.repos.d/*epages-$repo* 2>/dev/null) ; do
      sed -i -r "s,/[0-9]+\.[0-9]+-$repo,/$UPDATE_VER-$repo," $i
    done
  done
fi

if [[ -n $SUSE_REQ ]] ; then
  eval rpm -hiv $@ --test --prefix $EPAGES_PARENT $SUSE_REQ >> /tmp/${BN%.sh}.~$$~ 2>&1
fi

if [[ -n $REQ ]] ; then
  eval rpm -hiv $@ --prefix $EPAGES_PARENT $REQ 2>&1 | tee -a /tmp/${BN%.sh}.~$$~
elif [[ -n $REQ ]] ; then
  for i in $REQ ; do
    echo ${i#$EPAGES_RPMS_URL} >> /tmp/create-epages-repo.sh.~$$~
  done
fi

# -----------------------------------------------------------------------------
# find missing capabilities
# -----------------------------------------------------------------------------

unset REQ

# some capabilities needed for epages packages are missing
for cap in $(awk '/ is needed /{print $1}' /tmp/${BN%.sh}.~$$~ 2>/dev/null) ; do

  case $cap in
    apache2|apache2-worker|audit-libs|bind-utils|coreutils|graphviz|nodejs) REQ="$REQ $cap" ;;
    gawk|httpd|libstdc++|make|mod_ssl|net-tools|pam|sed|file|gd) REQ="$REQ $cap" ;;
    procps|python-urlgrabber|rpm-python|java-1.6.0-openjdk) REQ="$REQ $cap" ;;
    dejavu-*-fonts) REQ="$REQ $cap" ;;
    jre-openjdk) REQ="$REQ java-1.6.0-openjdk" ;;
    jre-sun) REQ="$REQ java-1_4_2-sun" ;;
    jre-ibm) REQ="$REQ java-1_6_0-ibm" ;;
  esac

  K="$KERNEL-$ARCH"
  case $K in
    2.6.32r-x86_64) I86=.i686 ;;
    2.6.18r-x86_64) I86=.i386 ;;
    *s-*) I86=.i586 ;;
    *) unset I86 ;;
  esac

  case $K in
    *r-*)
    X11=$(basename ${cap%.so.?})
    case $cap in
      libc.so.6) REQ="$REQ glibc" ;;
      libaio.so.1) REQ="$REQ libaio" ;;
      libasound.so.2) REQ="$REQ alsa-lib$I86" ;;
      /lib/libgcc_s.so.1) REQ="$REQ libgcc$I86" ;;
      /lib64/libgcc_s.so.1) REQ="$REQ libgcc" ;;
      /lib/libpam.so.0) REQ="$REQ pam$I86" ;;
      /lib64/libpam.so.0) REQ="$REQ pam" ;;
      /lib/libaudit.so.0) [ $K = ${K#2.6.32} ] || REQ="$REQ audit-libs$I86" ;;
      /lib64/libaudit.so.0) [ $K = ${K#2.6.32} ] || REQ="$REQ audit-libs" ;;
      /usr/lib/libstdc++.so.5) REQ="$REQ  compat-libstdc++-33$I86" ;;
      /usr/lib64/libstdc++.so.5) REQ="$REQ  compat-libstdc++-33" ;;
      /usr/lib/libstdc++.so.6) REQ="$REQ libstdc++$I86" ;;
      /usr/lib64/libstdc++.so.6) REQ="$REQ libstdc++" ;;
      /usr/lib/libstdc++-libc6.2-2.so.3) REQ="$REQ compat-libstdc++-296" ;;
      /sbin/chkconfig) REQ="$REQ chkconfig" ;;
      /usr/bin/crontab) [ $K = ${K#2.6.18} ] && REQ="$REQ cronie" || REQ="$REQ vixie-cron" ;;
      /usr/bin/getconf) REQ="$REQ glibc-common" ;;
      /usr/bin/mysql) REQ="$REQ mysql" ;;
      /usr/bin/mysqld_safe) REQ="$REQ mysql-server" ;;
      /usr/lib/libjpeg.so.62) REQ="$REQ libjpeg$I86" ;;
      /usr/lib/libpng12.so.0) REQ="$REQ libpng$I86" ;;
      /usr/sbin/sendmail) REQ="$REQ sendmail" ;;
      libgssapi_krb5.so.2) REQ="$REQ krb5-libs$I86" ;;
      */libz.so.0) REQ="$REQ zlib$I86" ;;
      /lib/libexpat.so.*) REQ="$REQ expat$I86" ;;
      /usr/lib/libexpat.so.0) REQ="$REQ compat-expat1$I86" ;;
      /usr/lib/libgdbm.so.2) REQ="$REQ gdbm$I86" ;;
      gnupg) [ $K = ${K#2.6.32} ] && REQ="$REQ gnupg" ; [ $K = ${K#2.6.9} ] && REQ="$REQ gnupg2" ;;
      /usr/lib/libxml2.so.2) REQ="$REQ libxml2$I86" ;;
      /usr/lib/libexslt.so.0) REQ="$REQ libxslt$I86" ;;
      /usr/lib/libfreetype.so.6) REQ="$REQ freetype$I86" ;;
      /usr/lib/libfontconfig.so.1) REQ="$REQ fontconfig$I86" ;;
      libX*.so.?|libICE.so.6|libSM.so.6) REQ="$REQ $X11" ;;
      /usr/lib/libX*.so.?) REQ="$REQ $X11$I86" ;;
      /usr/lib/libssl.so.6) [ $K = ${K#2.6.32} ] && REQ="$REQ openssl$I86" || REQ="$REQ openssl098e$I86" ;;
      /usr/lib/libssl.so.10|/lib/libssl.so.6) REQ="$REQ openssl$I86" ;;
      /usr/lib/libgd.so.2) REQ="$REQ gd$I86" ;;
      /lib/libuuid.so.1) [ $K = ${K#2.6.18} ] && REQ="$REQ libuuid$I86" || REQ="$REQ e2fsprogs-libs$I86" ;;
      /usr/lib/libltdl.so.7) REQ="$REQ libtool-ltdl$I86" ;;
    esac ;;
    *s-*)
    [[ $K != ${K#2.6.16} || $cap = libXi.so.6 || $cap = libXtst.so.6 ]] && X11=xorg-x11-libs || X11=xorg-x11-$(basename ${cap%.so.?})
    case $cap in
      libc.so.6) REQ="$REQ glibc" ;;
      libaio.so.1) REQ="$REQ libaio" ;;
      libasound.so.2) [ $K != ${K#2.6.16} ] && REQ="$REQ alsa" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libasound2 libasound2-32bit" || REQ="$REQ libasound2"; } ;;
      /lib/libgcc_s.so.1) [ $K != ${K#2.6.16} ] && REQ="$REQ libgcc" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libgcc43-32bit" || REQ="$REQ libgcc43"; } ;;
      /lib64/libgcc_s.so.1) [ $K != ${K#2.6.16} ] && REQ="$REQ libgcc" || REQ="$REQ libgcc43" ;;
      /lib/libpam.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ pam-32bit" || REQ="$REQ pam" ;;
      /lib64/libpam.so.0) REQ="$REQ pam" ;;
      /lib/libaudit.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ audit-libs-32bit" || REQ="$REQ audit-libs" ;;
      /lib64/libaudit.so.0) REQ="$REQ audit-libs" ;;
      /usr/lib/libstdc++.so.5) [ $K != ${K#2.6.16} ] && REQ="$REQ compat-libstdc++" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libstdc++33-32bit" || REQ="$REQ libstdc++33"; } ;;
      /usr/lib64/libstdc++.so.5) [ $K != ${K#2.6.16} ] && REQ="$REQ compat-libstdc++" || REQ="$REQ libstdc++33" ;;
      /usr/lib/libstdc++.so.6) [ $K != ${K#2.6.16} ] && REQ="$REQ libstdc++" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libstdc++43-32bit" || REQ="$REQ libstdc++43"; } ;;
      /usr/lib64/libstdc++.so.6) [ $K != ${K#2.6.16} ] && REQ="$REQ libstdc++" || REQ="$REQ libstdc++43" ;;
      /usr/lib/libstdc++-libc6.2-2.so.3) [ $K != ${K%x86_64} ] && REQ="$REQ compat-32bit"  || REQ="$REQ compat" ;;
      /sbin/chkconfig) REQ="$REQ aaa_base" ;;
      /usr/bin/crontab) REQ="$REQ cron" ;;
      /usr/bin/getconf) REQ="$REQ glibc" ;;
      /usr/bin/mysql) REQ="$REQ mysql-client" ;;
      /usr/bin/mysqld_safe) REQ="$REQ mysql" ;;
      /usr/lib/libjpeg.so.62) [ $K != ${K%x86_64} ] && REQ="$REQ libjpeg-32bit" || REQ="$REQ libjpeg" ;;
      /usr/lib/libpng12.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ libpng-32bit" || REQ="$REQ libpng" ;;
      /usr/sbin/sendmail) REQ="$REQ postfix" ;;
      libgssapi_krb5.so.2) [ $K != ${K%x86_64} ] && REQ="$REQ krb5-32bit" || REQ="$REQ krb5" ;;
      */libz.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ zlib-32bit"  || REQ="$REQ zlib" ;;
      /lib/libexpat.so.1) [ $K != ${K%x86_64} ] && REQ="$REQ libexpat1-32bit"  || REQ="$REQ libexpat1" ;;
      /usr/lib/libexpat.so.1) [ $K != ${K%x86_64} ] && REQ="$REQ expat-32bit"  || REQ="$REQ expat" ;;
      gnupg) [ $K = ${K#2.6.16} ] && REQ="$REQ gpg2" || REQ="$REQ gpg" ;;
      /usr/lib/libxml2.so.2) REQ="$REQ libxml2$I86" ;;
      /usr/lib/libexslt.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ libxslt-32bit" || REQ="$REQ libxslt" ;;
      /usr/lib/libfreetype.so.6) [ $K != ${K%x86_64} ] && REQ="$REQ freetype2-32bit" || REQ="$REQ freetype2" ;;
      /usr/lib/libfontconfig.so.1) [ $K != ${K%x86_64} ] && REQ="$REQ fontconfig-32bit" || REQ="$REQ fontconfig" ;;
      libX*.so.?|libICE.so.6|libSM.so.6) REQ="$REQ $X11" ;;
      /usr/lib/libX*.so.?) [ $K != ${K%x86_64} ] && REQ="$REQ $X11-32bit" || REQ="$REQ $X11" ;;
      /usr/lib/libssl.so.0.9.8) [ $K != ${K#2.6.16} ] && {
        [ $K != ${K%x86_64} ] && REQ="$REQ openssl-32bit" || REQ="$REQ openssl"; } || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libopenssl0_9_8-32bit" || REQ="$REQ libopenssl0_9_8"; } ;;
      /lib/libuuid.so.1) [ $K != ${K#2.6.16} ] && {
        [ $K != ${K%x86_64} ] && REQ="$REQ e2fsprogs-32bit" || REQ="$REQ e2fsprogs"; } || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libuuid1-32bit" || REQ="$REQ libuuid1"; } ;;
      /usr/lib/libltdl.so.7) [ $K != ${K%x86_64} ] && REQ="$REQ libltdl7-32bit" || REQ="$REQ libltdl7" ;;
    esac ;;
  esac

done

edit_repos() {
  local cap="graphviz nodejs"
  local DIST=rhel
  [[ $KERNEL = ${KERNEL%s} ]] || { cap=nodejs; DIST=sles; }

  for i in $cap ; do
    [[ $REQ = ${REQ#*$i} ]] && continue
    [[ -f /etc/yum.repos.d/$i-$DIST.repo ]] || continue
    awk '{if (/^enabled=/ && !X) {sub(/^enabled=.*/,"enabled=1");X=1}; print;}' \
    /etc/yum.repos.d/$i-$DIST.repo > /etc/yum.repos.d/$i-$DIST.repo.rpmsave
    if [[ $KERNEL = 2.6.27s && $i-$DIST = nodejs-sles ]] ; then
      sed -i 's,_SP1,,g' /etc/yum.repos.d/$i-$DIST.repo.rpmsave
    fi
    cp -p /etc/yum.repos.d/$i-$DIST.repo.rpmsave /etc/yum.repos.d/$i-$DIST.repo
  done
}

restore_repos() {
  rpm -e epages-release 2>/dev/null
  for i in $(ls /etc/yum.repos.d/{nodejs,graphviz}*.repo.rpmsave 2>/dev/null) ; do
    mv $i ${i%.rpmsave}
  done
  rm -f /etc/yum.repos.d/*epages-{setup,update,empty}*
}

[ $KERNEL = ${KERNEL%s} ] && INSTALLER=yum || INSTALLER=zypper
REQ=$(echo $REQ | tr ' ' '\n' | sort | uniq | tr '\n' ' ' | sed 's,^ *,,;s, *$,,')
edit_repos

if [[ -n "$REQ" ]] ; then
  restore_repos
  ADD_ZYPPER_REPOS=/etc/yum.repos.d/nodejs-sles.repo
  echo
  echo '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  echo "First install missing RPM packages by executing:"
  echo "$INSTALLER install $REQ"
  echo "Then re-run:"
  echo "$0$OWN_REPO $@"
  echo '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  echo
  RET=1
else
  if [[ -n $RELEASE_RPM_INSTALLED && ( -n $OWN_REPO || -z $INSTALL_RELEASE_RPM ) ]] && cd /etc/yum.repos.d ; then
    sed -i -r "s,^(\s*baseurl\b).*,\1 = $EPAGES_RPMS_URL/$BASE_VER/\$basearch," epages.repo
    sed -i -r "s,^(\s*baseurl\b).*,\1 = $EPAGES_RPMS_URL/$SETUP_VER-setup," epages-setup.repo
    sed -i -r "s,^(\s*baseurl\b).*,\1 = $EPAGES_RPMS_URL/$UPDATE_VER-update," epages-update.repo
    sed -i -r "s,^(\s*baseurl\b).*,\1 = $EPAGES_RPMS_URL/$UPDATE_VER-empty," epages-empty.repo
    sed -i -r "s,^(\s*baseurl\b).*,\1 = $EPAGES_RPMS_URL/kernel/$KERNEL," epages-kernel.repo

    echo "$EPAGES_RPMS_URL/$BASE_VER/\$ARCH" > mirrors-epages
    echo "$EPAGES_RPMS_URL/$SETUP_VER-setup" > mirrors-epages-setup
    echo "$EPAGES_RPMS_URL/$UPDATE_VER-update" > mirrors-epages-update
    echo "$EPAGES_RPMS_URL/$UPDATE_VER-empty" > mirrors-epages-empty
    echo "$EPAGES_RPMS_URL/kernel/$KERNEL" > mirrors-epages-kernel-$KERNEL

    [[ -n $DEV_REPO ]] && sed -i -r "s,^(\s*enabled\b).*,\1 = 0," epages-setup.repo

    if [[ -z $INSTALL_RELEASE_RPM && $INSTALLER = yum ]] ; then
      pgrep yum | grep -q . || yum clean metadata
    elif [[ -z $INSTALL_RELEASE_RPM ]] ; then
      pgrep zypper | grep -q . || zypper clean -m
    fi

  fi

  unset RET
  if [[ -n $RELEASE_RPM_INSTALLED && -z $EPAGES_INSTALLED && -n $EPAGES_INSTALL_VERSION ]] ; then
      
    # check if selected $EPAGES_INSTALL_VERSION exists
    EXISTS=$(ini-config.sh -f /etc/yum.repos.d/epages-archive.repo -s epages-$EPAGES_INSTALL_VERSION -k baseurl -g)
    if [[ -n $EXISTS ]] ; then
      EXISTS=$(rpm -qp "$EXISTS/epages-6-0.i386.rpm" --qf '[%{VERSION}\n]' 2>/dev/null | egrep '^[0-9.]+$')
      [[ -n $EXISTS ]] || RET=2
    else
      RET=1
    fi
    if [[ -n $RET ]] ; then
      rpm -e epages-release 2>/dev/null
      rm -f /etc/yum.repos.d/*epages-{setup,update,empty}*
      echo
      echo '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
      echo 'ERROR:'
      echo "No epages RPM found for EPAGES_INSTALL_VERSION=$EPAGES_INSTALL_VERSION in:"
      echo "  http://www.epages.com/archive/RPMS/$EPAGES_INSTALL_VERSION/epages-6-0.i386.rpm"
      echo '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
      echo
    else 
      BASEURL=ftp://epages-software.de/pub/products/RPMS/${EPAGES_INSTALL_VERSION%.*}
      for i in "" -setup -update -empty ; do
        ini-config.sh -f /etc/yum.repos.d/epages$i.repo -s epages$i -k enabled -v 0 -i || exit 1
        ini-config.sh -f /etc/yum.repos.d/epages$i.repo -s epages$i -k baseurl -v $BASEURL${i:-/\$basearch} -i || exit 1
        sed -i "s,/RPMS/.*,/RPMS/${EPAGES_INSTALL_VERSION%.*}${i:-/\$ARCH}," /etc/yum.repos.d/mirrors-epages$i
      done
    fi
  fi

  if [[ -n $DEVEL_VER ]] && rpm --quiet -q epages-common ; then
    ini-config.sh -f /etc/yum.repos.d/epages.repo -s epages -k enabled -v 0 -i || exit 1
  fi

  [[ -z $RET ]] && { grep -q 'error:' /tmp/${BN%.sh}.~$$~ 2>/dev/null && RET=1 || RET=0; }
fi

if cd /etc/zypp/repos.d 2>/dev/null ; then
  for i in $(ls $ADD_ZYPPER_REPOS 2>/dev/null) ; do
    [[ $(find ${i##*/} -printf "%l" 2>/dev/null) = $i ]] && continue
    ln -s $i .
  done
fi

exit $RET
