#!/bin/sh

########################################################################
# SET ENVIRONMENT FOR EPAGES
########################################################################

check_variables () {
  unset UNSET
  for i in $@ ; do
    if [ -z "`eval echo '$'$i`" ] ; then
      UNSET="$UNSET $i"
    fi
  done
  if [ -n "$UNSET" ] ; then
    echo "Following required variables must be set:"
    echo " $UNSET"
    echo "You might do this by running:"
    echo "  . /etc/default/epages6"
    exit 1
  fi
}

check_variables EPAGES_PROJECT EPAGES_PERL EPAGES_CARTRIDGES EPAGES_CONFIG

EPAGES_INITD=/etc/init.d/$EPAGES_PROJECT

if [ ! -x $EPAGES_INITD ] ; then
  echo "Can't execute $EPAGES_INITD."
  exit 1
fi

DN="`dirname $0`"
DN=`cd "$DN" ; pwd` || exit 1

PERL=${PERL:-"$EPAGES_PERL/bin/perl"}
if [ "x`\"$PERL\" -e 'print 1'`" != "x1" ] ; then
  echo "Program perl not found in '$PERL'."
  exit 1
fi

runuser -c true 2>/dev/null && SU_EXE=runuser || SU_EXE=su

TESTSOAP_PL="$EPAGES_CARTRIDGES"/DE_EPAGES/ShopConfiguration/Scripts/testHttp.pl
for FILE in "$TESTSOAP_PL" ; do
  if [ ! -f "$FILE" ] ; then
    echo "File not found: $FILE"
    exit 1
  fi
done

########################################################################
# SHELL FUNCTIONS
########################################################################

exit_setup()
{
  [ $1 -ne 0 ] && $EPAGES_INITD stop_service
  exit $1
}

CONNECT_TO_DATABASE='
use DE_EPAGES::Core::API::Error qw ( Error GetError ExistsError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( ConnectSite );
RunScript( Sub => \&Test );
sub Test { eval { ConnectSite("Backup"); }; Error(GetError()) if ExistsError(); }
'
connect_to_database()
{
  echo "Testing database connectivity"
  echo "   Perl: DE_EPAGES::Database::API::Connection::ConnectSite('Backup')"
  echo -n "   Status: "

  for i in 1 2 3 4 5 6 7 8 9 ; do
    ERROR=0
    "$PERL" -e "$CONNECT_TO_DATABASE" >/dev/null 2>&1 && break
    ERROR=1
    sleep 2
    echo -n "."
  done
  [ "$ERROR" = 1 ] && echo "failed (see error.log)" || echo works
  return $ERROR
}

########################################################################
# MAIN
########################################################################

echo "Installing epages cartridges."
echo "-----------------------------"
echo "(Please be patient, this may take some minutes.)"

ls "$EPAGES_CONFIG/epages.conf" >/dev/null || { echo No ePages-ready web server found, exiting.; exit_setup $?; }
PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin
WEB_HOST=`ini-config.sh -f "$EPAGES_CONFIG/epages.conf" -s "DE_EPAGES::Store" -k DefaultWebServiceProxy -g 2>/dev/null | awk -F/ '{print $3}'`
[ -z "$WEB_HOST" ] && { echo "$EPAGES_CONFIG/epages.conf: Can't read key DefaultWebServiceProxy in section DE_EPAGES::Store, exiting."; exit_setup $?; }
"$PERL" "$TESTSOAP_PL" -verbosity 1 -host "$WEB_HOST" || exit_setup $?

cd $EPAGES_CARTRIDGES/DE_EPAGES || exit_setup $?
"$PERL" Makefile.PL || exit_setup $?
chown ep_appl:epages makefile || exit_setup $?

for i in ${EPAGES_BUSINESS_UNITS:-Site Store} ; do
  make drop_db STORE=${i#Flex} STORE_TYPE=$i >/dev/null 2>&1
done

. /etc/sysconfig/epages6

if [[ -x /etc/init.d/epagesj ]] ; then
  /etc/init.d/epagesj restart
  EPAGESJ_PORT=${EPAGES_INSTALL_EPAGESJ_PORT:-8088}
  i=0
  while [[ $i -lt 30 ]] ; do
    netstat -tlpn | grep -q ":$EPAGESJ_PORT " && break
    i=$(($i + 1))
    sleep 1
  done
fi

eval make install $EPAGES_INSTALL_MAKE_ARGS || exit_setup $?

rm -f $EPAGES_CONFIG/appsrv.ini
echo "-----------------------------"
echo "Installing cartridges finished."

exit_setup 0

########################################################################
# END
########################################################################
