#!/bin/env bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBP6r9rl5jMabHSgyARAp/FAJ9IJsJE62cDDQfiyU9gsN+dj38tUgCfX8lR
#j945YoLQjOmqzwqqFL8+WXA=
#=z+wC
#-----END PGP SIGNATURE-----

BN=`basename $0`

[ "x-show" = "x$1" ] && SHOW_REWRITE=1 || unset SHOW_REWRITE

if [ $# -gt 0 -a "x-show" != "x$1" ] ; then
  echo "Usage:"
  echo " [ENVVAR=VAL] $0 [-show]"
  echo "ENVVAR:"
  echo " URLREWRITE_WEBINTERFACE_CONF=FILE (default: \$EPAGES_CONFIG/WebInterface.conf)"
  echo " EPAGES_CLOSE_WEBSITE={0|1} (0 - open; 1 - close; default: empty)"
  echo " EPAGES_IGNORE_REWRITE_CONF=1 (no EPAGES_REWRITE_CONF changes; default: empty)"
  echo " WEBSRV_PROCESS_NAME=NAME (used together with -show; default: httpd)"
  echo "Options:"
  echo " -show : show the RewriteLog"
  echo "Example:"
  echo " WEBSRV_PROCESS_NAME=httpd-worker \$EPAGES/bin/$BN -show"
  exit 1
fi

# die
die() {
  [ -n "$SHOW_REWRITE" ] && echo -e "$BN: $1"
  exit ${2:-0}
}

# quote_host
quote_host() {
  echo "$1" | sed 's/\(\.\)/\\\1/g;s,?,.,g;s,\*,.\*,g;'
}

# exit if nothing to do
[[ -z $EPAGES_CLOSE_WEBSITE && -n $EPAGES_IGNORE_REWRITE_CONF ]] && exit 0

EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}
. /etc/default/"$EPAGES_PROJECT"

# exit if file WebInterface.conf isn't readable
URLREWRITE_WEBINTERFACE_CONF=${URLREWRITE_WEBINTERFACE_CONF:-"$EPAGES_CONFIG/WebInterface.conf"}
[ -r "$URLREWRITE_WEBINTERFACE_CONF" ] || die "Can't read: $URLREWRITE_WEBINTERFACE_CONF" 1

# exit if file ini-config.sh isn't executable
PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin
type ini-config.sh >/dev/null 2>&1 || die "Can't execute ini-config.sh" 1

timeout_test -x "$PERL" || PERL=$(type -p perl)

# read configuration values
[ -n "$SHOW_REWRITE" ] && echo -e "\nread  conf: $URLREWRITE_WEBINTERFACE_CONF"
RW_EPAGES=$(ini-config.sh -f "$URLREWRITE_WEBINTERFACE_CONF" -s URLRewrite -k epages -g)
RW_EPAGES=${RW_EPAGES:-/epages/}
RW_BASEURI=$(ini-config.sh -f  "$URLREWRITE_WEBINTERFACE_CONF" -s URLRewrite -k BaseURI -g)
RW_EX_PREFIX=$(ini-config.sh -f  "$URLREWRITE_WEBINTERFACE_CONF" -s URLRewrite -k ExceptionPrefix -g)
RW_EX_REGEX=$(ini-config.sh -f  "$URLREWRITE_WEBINTERFACE_CONF" -s URLRewrite -k ExceptionRegex -g)
RW_EX_HOST=$(ini-config.sh -f  "$URLREWRITE_WEBINTERFACE_CONF" -s URLRewrite -k ExceptionHost -g)
LOCAL_HOSTS=$(ini-config.sh -f  "$URLREWRITE_WEBINTERFACE_CONF" -s URLRewrite -k LocalHosts -g)
CLOSED_PAGE=$(ini-config.sh -f  "$URLREWRITE_WEBINTERFACE_CONF" -s URLRewrite -k StoreClosedPage -g)
CLOSED_PAGE=${CLOSED_PAGE:-"$EPAGES_WEBROOT/StoreClosed.html"}
CLOSED_PREFIX=$(ini-config.sh -f  "$URLREWRITE_WEBINTERFACE_CONF" -s URLRewrite -k StoreClosedPrefix -g)
CLOSED_PREFIX=${CLOSED_PREFIX:-/WebRoot/Icons/}
CLOSED_CONF=$(ini-config.sh -f  "$URLREWRITE_WEBINTERFACE_CONF" -s URLRewrite -k StoreClosedConf -g)
REWRITE_CONF=$EPAGES_REWRITE_CONF
[[ -n $EPAGES_CLOSE_WEBSITE ]] && EPAGES_REWRITE_CONF=${CLOSED_CONF:-$EPAGES_REWRITE_CONF}

# exit if EPAGES_REWRITE_CONF empty
[ -z "$EPAGES_REWRITE_CONF" ] && die "environment variable empty: EPAGES_REWRITE_CONF"

# exit if nothing to do
[[ $EPAGES_CLOSE_WEBSITE = 1 && -z $LOCAL_HOSTS ]] && die "EPAGES_CLOSE_WEBSITE = 1 and LOCAL_HOSTS empty"

# exit if directory doesn't exist
DN=$(dirname "$EPAGES_REWRITE_CONF")
[ -d "$DN" ] || die "Directory for EPAGES_REWRITE_CONF must exist:\n$DN: No such file or directory"

# create $EPAGES_REWRITE_CONF unless exists
if [ ! -f "$EPAGES_REWRITE_CONF" ] ; then
  touch "$EPAGES_REWRITE_CONF" || die "Can't write: $EPAGES_REWRITE_CONF"
  if [ -w /etc/passwd ] ; then
    OWNER=${EPAGES_WEBGROUP##*:}
    [ -z "$OWNER" ] && OWNER=ep_web || OWNER=ep_web:$OWNER
    [ -n "$OWNER" ] && chown $OWNER "$EPAGES_REWRITE_CONF"
  fi
fi

# SHOW_REWRITE
if [ -n "$SHOW_REWRITE" ] ; then
  # exit if perl doesn't work
  $PERL -MLWP::Simple -e 1 || exit 1

  # exit if web server is not running, else set WEBSRV_PID
  if [ -n "$WEBSRV_PROCESS_NAME" ] ; then
    set -- `pgrep -u root $WEBSRV_PROCESS_NAME`
  else
    set -- `pgrep -u root httpd-epages`
    [ $# -eq 0 ] && set -- `pgrep -u root httpd`
  fi
  if [ $# -eq 0 ] ; then
    echo "$BN: ${WEBSRV_PROCESS_NAME:-httpd}: no such process found"
    echo "  Make sure that a) the web server is running and"
    echo "  b) the name of the web server (${WEBSRV_PROCESS_NAME:-httpd}) is correct."
    echo "  If the name of the web server differs, run:"
    echo "    WEBSRV_PROCESS_NAME=<MY_NAME> \$EPAGES/bin/$BN -show"
    echo "  Example:"
    echo "    WEBSRV_PROCESS_NAME=httpd-worker \$EPAGES/bin/$BN -show"
    exit 1
  elif [ $# -eq 1 ] ; then
    WEBSRV_PID=$1
    ps -p${WEBSRV_PID// /,} -ouid_hack,pid,stime,args
  else
    echo "Various web server processes found:"
    WEBSRV_PID=$@
    i=CHOOSE ; ps -p${WEBSRV_PID// /,} -ouid_hack,pid,stime,args | while read line ; do
      echo "$line [$i]"; i=`expr $i + 1 2>/dev/null || echo 1`
    done
    echo -n "Choose the right web server process (1 .. $#): "
    declare -i ans ; read ans
    if [ $ans -lt 1 -o $ans -gt $# ] ; then
      echo "$BN: Aborted."
      exit 0
    fi
    WEBSRV_PID=`eval echo '$'$ans`
  fi
fi

# write empty $EPAGES_REWRITE_CONF
if [[ ( -z "$RW_EX_PREFIX$RW_EX_REGEX$RW_EX_HOST" && -z ${EPAGES_CLOSE_WEBSITE#0} )
      || ( $EPAGES_CLOSE_WEBSITE = 0 && -n $CLOSED_CONF && $REWRITE_CONF != $CLOSED_CONF ) ]] ; then
  cat /dev/null > "$EPAGES_REWRITE_CONF"
  die "$URLREWRITE_WEBINTERFACE_CONF: No rewrite instruction found"
fi
EPAGES_CLOSE_WEBSITE=${EPAGES_CLOSE_WEBSITE#0}

# write new $EPAGES_REWRITE_CONF
rm -f "$EPAGES_REWRITE_CONF".new

# EPAGES_CLOSE_WEBSITE: add Alias to $EPAGES_REWRITE_CONF
if [ -n "$EPAGES_CLOSE_WEBSITE" ] ; then
  echo Alias /Maintainance/ \"$(dirname "$CLOSED_PAGE")/\" > "$EPAGES_REWRITE_CONF".new || exit 1
fi

# write start tag
echo -e '<IfModule rewrite_module>\nRewriteEngine on' >> "$EPAGES_REWRITE_CONF".new || exit 1
[ -n "$SHOW_REWRITE" ] && echo -e 'RewriteLogLevel 2\nRewriteLog "/tmp/EPAGES_SHOW_REWRITE.log.~0815~"' >> "$EPAGES_REWRITE_CONF".new

# EPAGES_CLOSE_WEBSITE: edit $EPAGES_REWRITE_CONF
if [ -n "$EPAGES_CLOSE_WEBSITE" ] ; then
  for i in localhost 127.0.0.1 ::1 $LOCAL_HOSTS ; do
    echo 'RewriteCond %{REMOTE_HOST} !^'"$(quote_host "$i")"'$ [NC]' >> "$EPAGES_REWRITE_CONF".new
    echo 'RewriteCond %{REMOTE_ADDR} !^'"$(quote_host "$i")"'$ [NC]' >> "$EPAGES_REWRITE_CONF".new
  done
  for i in $CLOSED_PREFIX ; do
    echo 'RewriteCond %{REQUEST_URI} !^'"$i" >> "$EPAGES_REWRITE_CONF".new
  done
  STORE_CLOSED=/Maintainance/$(basename "$CLOSED_PAGE")
  cat <<EOF >> "$EPAGES_REWRITE_CONF".new
RewriteCond %{REQUEST_URI} !=$STORE_CLOSED
RewriteRule ^ - [R=503,L]
</IfModule>

ErrorDocument 503 $STORE_CLOSED
EOF
fi

# EPAGES_REWRITE_CONF: edit $EPAGES_REWRITE_CONF
if [ -z "$EPAGES_CLOSE_WEBSITE" ] ; then
  for i in $RW_EX_HOST ; do
    echo 'RewriteCond %{HTTP_HOST} !^'"$(quote_host "$i")"'$ [NC]' >> "$EPAGES_REWRITE_CONF".new
  done
  for i in $RW_EX_PREFIX ; do
    i="$(echo "$RW_BASEURI$i" | sed 's,///*,/,g')"
    echo 'RewriteCond %{REQUEST_URI} !^'"$i"'' >> "$EPAGES_REWRITE_CONF".new
  done
  for i in $RW_EX_REGEX ; do
    echo 'RewriteCond %{REQUEST_URI} !'"$i"'' >> "$EPAGES_REWRITE_CONF".new
  done
  echo 'RewriteRule ^'"${RW_BASEURI%/}"'/(.*)$ '"${RW_EPAGES%/}"'/.shorturl/?URI=$1 [PT,QSA]' >> "$EPAGES_REWRITE_CONF".new
  echo '</IfModule>' >> "$EPAGES_REWRITE_CONF".new
fi

# backup existing $EPAGES_REWRITE_CONF
if [ -n "$SHOW_REWRITE" ] ; then
  cp -p "$EPAGES_REWRITE_CONF" "$EPAGES_REWRITE_CONF".sofar
else
  cmp "$EPAGES_REWRITE_CONF" "$EPAGES_REWRITE_CONF".new >/dev/null 2>&1 \
  || cp -p "$EPAGES_REWRITE_CONF" "$EPAGES_REWRITE_CONF".~$(date +%s)~
fi
[ -n "$SHOW_REWRITE" ] && echo "write conf: $EPAGES_REWRITE_CONF"

# copy new $EPAGES_REWRITE_CONF
cat "$EPAGES_REWRITE_CONF".new > "$EPAGES_REWRITE_CONF" && rm -f "$EPAGES_REWRITE_CONF".new

# SHOW_REWRITE
if [ -n "$SHOW_REWRITE" -a -n "$WEBSRV_PID" ] ; then
  kill -HUP $WEBSRV_PID

  cat /dev/null > /tmp/EPAGES_SHOW_REWRITE.log.~0815~
  echo -e "\n$PERL -MLWP::Simple -e 'get \"http://127.0.0.1/TEST\" || get \"http://[::1]/TEST\"'"
  $PERL -MLWP::Simple -e 'get "http://127.0.0.1/TEST" || get "http://[::1]/TEST"'
  if awk -F\] '{sub(/- - /,"");sub(/\(2\) /,"");printf "%s]%s\n", $1, $4}' \
  < /tmp/EPAGES_SHOW_REWRITE.log.~0815~ && [ -w /etc/passwd  ] ; then
    GROUP=${EPAGES_WEBGROUP##*:}
    GROUP=${GROUP:-epages}
    chown ep_appl:$GROUP /tmp/EPAGES_SHOW_REWRITE.log.~0815~
    chmod 664 /tmp/EPAGES_SHOW_REWRITE.log.~0815~
  fi
  grep -q "/epages/\.shorturl/" /tmp/EPAGES_SHOW_REWRITE.log.~0815~ || {
    echo -e "\n$BN: ${WEBSRV_PROCESS_NAME:-httpd}: Rewrite failed."
  }
  mv "$EPAGES_REWRITE_CONF".sofar "$EPAGES_REWRITE_CONF"
  kill -HUP $WEBSRV_PID || {
    echo -e "\n$BN: Restart of web server failed.\nPlease start the web server by yourself."
  }
fi
