#!/bin/env bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBR8iRKl5jMabHSgyARAgpXAKCex9SX6oKIcrMSY/EJUa38kYr8uwCfW+XD
#dHr+h5sQWHv1oOv+rWjRgxM=
#=pCfP
#-----END PGP SIGNATURE-----

BN=`basename $0`

[ "x-v" = "x$1" ] && { VERBOSE=1; shift; } || unset VERBOSE

if [[ $1 =~ - ]] ; then
  echo "Usage:"
  echo "  $0 [-v] [IP...]"
  exit 1
fi

# exit if ePages environment is incomplete
if [ -z "$EPAGES" -o -z "$EPAGES_CONFIG" -o -z "$EPAGES_PROJECT" ] ; then
  EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}
  . /etc/default/"$EPAGES_PROJECT"
fi
if [ -z "$EPAGES" -o -z "$EPAGES_CONFIG" ] ; then
  if [ -n "$VERBOSE" ] ; then
    echo "$BN: ePages environment incomplete in /etc/default/\$EPAGES_PROJECT:"
    echo "  Correct the values of following environment variables:"
    echo "  EPAGES_PROJECT=$EPAGES_PROJECT"
    echo "  EPAGES=$EPAGES"
    echo "  EPAGES_CONFIG=$EPAGES_CONFIG"
  fi
  exit 0
fi

DEFAULT_SCRIPT=/etc/default/"$EPAGES_PROJECT"
WEBINTERFACE_CONF="$EPAGES_CONFIG"/WebInterface.conf
PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin

DEFAULT_SCRIPT=$(readlink -f "$DEFAULT_SCRIPT")

# exit if file /etc/default/epages6 isn't writeable
if [ ! -w "$DEFAULT_SCRIPT" ] ; then
  [ -n "$VERBOSE" ] && echo "$BN: Can't write $DEFAULT_SCRIPT"
  exit 1
fi

# exit if file ini-config.sh isn't executable
type ini-config.sh > /dev/null 2>&1 || {
  [ -n "$VERBOSE" ] && echo "$BN: Can't execute ini-config.sh"
  exit 1
}

if [[ $# -gt 0 ]] ; then
  LOCAL_IPS=$(ini-config.sh -G -k "LOCAL_IPS" -f "$DEFAULT_SCRIPT")
  LOCAL_IPS=$(echo ${LOCAL_IPS#*-i})
  LOCAL_IPS=${LOCAL_IPS:+,$LOCAL_IPS,}
  LOCAL_IPS=${LOCAL_IPS:-,}
  for i in $* ; do
    [[ $LOCAL_IPS =~ ,$i, ]] || LOCAL_IPS=$LOCAL_IPS$i,
  done

else

  # exit if file WebInterface.conf isn't readable
  if [ ! -r "$WEBINTERFACE_CONF" ] ; then
    [ -n "$VERBOSE" ] && echo "$BN: Can't read \$EPAGES_CONFIG/WebInterface.conf"
    exit 1
  fi

  LOCAL_IPS=,
  # find local ips in WebInterface.conf
  for i in $(ini-config.sh -g -f "$WEBINTERFACE_CONF") ; do
    u=$(echo $i | tr [a-z] [A-Z])
    [ $u = GLOBAL -o $u = WA-DEBUG -o $u = URLREWRITE ] && continue
    [ $i = ${i//\*/} ] || continue
    [ "$LOCAL_IPS" = "${LOCAL_IPS//,$i,/}" ] || continue
    LOCAL_IPS="$LOCAL_IPS$i,"
    for j in $(ini-config.sh -g -s "$i" -k "DEDICATED" -f "$WEBINTERFACE_CONF") ; do
      [ "$LOCAL_IPS" = "${LOCAL_IPS//,$j,/}" ] || continue
      LOCAL_IPS="$LOCAL_IPS$j,"
    done
  done
fi

LOCAL_IPS=$(echo ${LOCAL_IPS%,} | sed -r 's/,,+/,/g')
LOCAL_IPS=${LOCAL_IPS#,}
if [ -n "$LOCAL_IPS" ] ; then
  IP_BINDING=-b
  LOCAL_IPS="\"-i $LOCAL_IPS\""
else
  unset IP_BINDING
fi

# write new $DEFAULT_SCRIPT
sed "s/^\(LOCAL_IPS\)=.*/\1=$LOCAL_IPS/;s/^\(IP_BINDING\)=.*/\1=$IP_BINDING/" \
"$DEFAULT_SCRIPT" > "$DEFAULT_SCRIPT".new

# backup existing $DEFAULT_SCRIPT or exit
if cmp "$DEFAULT_SCRIPT" "$DEFAULT_SCRIPT".new >/dev/null 2>&1 ; then
  rm -f "$DEFAULT_SCRIPT".new
  [ -n "$VERBOSE" ] && echo "not changed: /etc/default/$EPAGES_PROJECT"
  exit 0
fi

cp -p "$DEFAULT_SCRIPT" "$DEFAULT_SCRIPT".~$(date +%s)~

# update existing $DEFAULT_SCRIPT
cat "$DEFAULT_SCRIPT".new > "$DEFAULT_SCRIPT" && rm -f "$DEFAULT_SCRIPT".new
[ -n "$VERBOSE" ] && echo "updated: /etc/default/$EPAGES_PROJECT"
